# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging
import urllib.parse

from dataclasses import dataclass
from typing import Any, Dict, List

from amundsen_common.models.dashboard import DashboardSummary, DashboardSummarySchema
from amundsen_common.models.popular_table import PopularTable, PopularTableSchema
from amundsen_common.models.table import Table, TableSchema
from amundsen_application.models.user import load_user, dump_user
from amundsen_application.config import MatchRuleObject
from flask import current_app as app
import re


@dataclass
class TableUri:
    database: str
    cluster: str
    schema: str
    table: str

    def __str__(self) -> str:
        return f"{self.database}://{self.cluster}.{self.schema}/{self.table}"

    @classmethod
    def from_uri(cls, uri: str) -> 'TableUri':
        parsed = urllib.parse.urlparse(uri)
        cluster, schema = parsed.netloc.rsplit('.', 1)
        return TableUri(
            database=parsed.scheme,
            cluster=cluster,
            schema=schema,
            table=parsed.path.lstrip('/')
        )


def marshall_table_partial(table_dict: Dict) -> Dict:
    """
    Forms a short version of a table Dict, with selected fields and an added 'key'
    :param table_dict: Dict of partial table object
    :return: partial table Dict

    TODO - Unify data format returned by search and metadata.
    """
    schema = PopularTableSchema(strict=True)
    # TODO: consider migrating to validate() instead of roundtripping
    table: PopularTable = schema.load(table_dict).data
    results = schema.dump(table).data
    # TODO: fix popular tables to provide these? remove if we're not using them?
    # TODO: Add the 'key' or 'id' to the base PopularTableSchema
    results['key'] = f'{table.database}://{table.cluster}.{table.schema}/{table.name}'
    results['last_updated_timestamp'] = None
    results['type'] = 'table'

    return results


def _parse_editable_rule(rule: MatchRuleObject,
                         schema: str,
                         table: str) -> bool:
    """
    Matches table name and schema with corresponding regex in matching rule
    :parm rule: MatchRuleObject defined in list UNEDITABLE_TABLE_DESCRIPTION_MATCH_RULES in config file
    :parm schema: schema name from Table Dict received from metadata service
    :parm table: table name from Table Dict received from metadata service
    :return: boolean which determines if table desc is editable or not for given table as per input matching rule
    """
    if rule.schema_regex and rule.table_name_regex:
        match_schema = re.match(rule.schema_regex, schema)
        match_table = re.match(rule.table_name_regex, table)
        return not (match_schema and match_table)

    if rule.schema_regex:
        return not re.match(rule.schema_regex, schema)

    if rule.table_name_regex:
        return not re.match(rule.table_name_regex, table)

    return True


def is_table_editable(schema_name: str, table_name: str, cfg: Any = None) -> bool:
    if cfg is None:
        cfg = app.config

    if schema_name in cfg['UNEDITABLE_SCHEMAS']:
        return False

    for rule in cfg['UNEDITABLE_TABLE_DESCRIPTION_MATCH_RULES']:
        if not _parse_editable_rule(rule, schema_name, table_name):
            return False

    return True


def marshall_table_full(table_dict: Dict) -> Dict:
    """
    Forms the full version of a table Dict, with additional and sanitized fields
    :param table_dict: Table Dict from metadata service
    :return: Table Dict with sanitized fields
    """

    schema = TableSchema(strict=True)
    # TODO: consider migrating to validate() instead of roundtripping
    table: Table = schema.load(table_dict).data
    results: Dict[str, Any] = schema.dump(table).data

    is_editable = is_table_editable(results['schema'], results['name'])
    results['is_editable'] = is_editable

    # TODO - Cleanup https://github.com/lyft/amundsen/issues/296
    #  This code will try to supplement some missing data since the data here is incomplete.
    #  Once the metadata service response provides complete user objects we can remove this.
    results['owners'] = [_map_user_object_to_schema(owner) for owner in results['owners']]
    readers = results['table_readers']
    for reader_object in readers:
        reader_object['user'] = _map_user_object_to_schema(reader_object['user'])

    columns = results['columns']
    for col in columns:
        # Set editable state
        col['is_editable'] = is_editable
        # If order is provided, we sort the column based on the pre-defined order
        if app.config['COLUMN_STAT_ORDER']:
            # the stat_type isn't defined in COLUMN_STAT_ORDER, we just use the max index for sorting
            col['stats'].sort(key=lambda x: app.config['COLUMN_STAT_ORDER'].
                              get(x['stat_type'], len(app.config['COLUMN_STAT_ORDER'])))

    # TODO: Add the 'key' or 'id' to the base TableSchema
    results['key'] = f'{table.database}://{table.cluster}.{table.schema}/{table.name}'
    # Temp code to make 'partition_key' and 'partition_value' part of the table
    results['partition'] = _get_partition_data(results['watermarks'])

    # We follow same style as column stat order for arranging the programmatic descriptions
    prog_descriptions = results['programmatic_descriptions']
    results['programmatic_descriptions'] = _convert_prog_descriptions(prog_descriptions)

    return results


def marshall_dashboard_partial(dashboard_dict: Dict) -> Dict:
    """
    Forms a short version of dashboard metadata, with selected fields and an added 'key'
    and 'type'
    :param dashboard_dict: Dict of partial dashboard metadata
    :return: partial dashboard Dict
    """
    schema = DashboardSummarySchema(strict=True)
    dashboard: DashboardSummary = schema.load(dashboard_dict).data
    results = schema.dump(dashboard).data
    results['type'] = 'dashboard'
    # TODO: Bookmark logic relies on key, opting to add this here to avoid messy logic in
    # React app and we have to clean up later.
    results['key'] = results.get('uri', '')
    return results


def marshall_dashboard_full(dashboard_dict: Dict) -> Dict:
    """
    Cleanup some fields in the dashboard response
    :param dashboard_dict: Dashboard response from metadata service.
    :return: Dashboard dictionary with sanitized fields, particularly the tables and owners.
    """
    # TODO - Cleanup https://github.com/lyft/amundsen/issues/296
    #  This code will try to supplement some missing data since the data here is incomplete.
    #  Once the metadata service response provides complete user objects we can remove this.
    dashboard_dict['owners'] = [_map_user_object_to_schema(owner) for owner in dashboard_dict['owners']]
    dashboard_dict['tables'] = [marshall_table_partial(table) for table in dashboard_dict['tables']]
    return dashboard_dict


def _convert_prog_descriptions(prog_descriptions: List = None) -> Dict:
    """
    Apply the PROGRAMMATIC_DISPLAY configuration to convert to the structure.
    :param prog_descriptions: A list of objects representing programmatic descriptions
    :return: A dictionary with organized programmatic_descriptions
    """
    left = []  # type: List
    right = []  # type: List
    other = prog_descriptions or []  # type: List
    updated_descriptions = {}

    if prog_descriptions:
        # We want to make sure there is a display title that is just source
        for desc in prog_descriptions:
            source = desc.get('source')
            if not source:
                logging.warning("no source found in: " + str(desc))

        # If config is defined for programmatic disply we organize and sort them based on the configuration
        prog_display_config = app.config['PROGRAMMATIC_DISPLAY']
        if prog_display_config:
            left_config = prog_display_config.get('LEFT', {})
            left = [x for x in prog_descriptions if x.get('source') in left_config]
            left.sort(key=lambda x: _sort_prog_descriptions(left_config, x))

            right_config = prog_display_config.get('RIGHT', {})
            right = [x for x in prog_descriptions if x.get('source') in right_config]
            right.sort(key=lambda x: _sort_prog_descriptions(right_config, x))

            other_config = dict(filter(lambda x: x not in ['LEFT', 'RIGHT'], prog_display_config.items()))
            other = list(filter(lambda x: x.get('source') not in left_config and x.get('source')
                                not in right_config, prog_descriptions))
            other.sort(key=lambda x: _sort_prog_descriptions(other_config, x))

    updated_descriptions['left'] = left
    updated_descriptions['right'] = right
    updated_descriptions['other'] = other
    return updated_descriptions


def _sort_prog_descriptions(base_config: Dict, prog_description: Dict) -> int:
    default_order = len(base_config)
    prog_description_source = prog_description.get('source')
    config_dict = base_config.get(prog_description_source)
    if config_dict:
        return config_dict.get('display_order', default_order)
    return default_order


def _map_user_object_to_schema(u: Dict) -> Dict:
    return dump_user(load_user(u))


def _get_partition_data(watermarks: Dict) -> Dict:
    if watermarks:
        high_watermark = next(filter(lambda x: x['watermark_type'] == 'high_watermark', watermarks))
        if high_watermark:
            return {
                'is_partitioned': True,
                'key': high_watermark['partition_key'],
                'value': high_watermark['partition_value']
            }
    return {
        'is_partitioned': False
    }



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0


class MailClientNotImplemented(Exception):
    """
    An exception when Mail Client is not implemented
    """
    pass



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Tuple
import logging

from flask import Flask, render_template
import jinja2
import os


ENVIRONMENT = os.getenv('APPLICATION_ENV', 'development')
LOGGER = logging.getLogger(__name__)


def init_routes(app: Flask) -> None:
    app.add_url_rule('/healthcheck', 'healthcheck', healthcheck)
    app.add_url_rule('/', 'index', index, defaults={'path': ''})  # also functions as catch_all
    app.add_url_rule('/<path:path>', 'index', index)  # catch_all


def index(path: str) -> Any:
    try:
        return render_template("index.html", env=ENVIRONMENT)  # pragma: no cover
    except jinja2.exceptions.TemplateNotFound as e:
        LOGGER.error("index.html template not found, have you built the front-end JS (npm run build in static/?")
        raise e


def healthcheck() -> Tuple[str, int]:
    return '', 200  # pragma: no cover



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
from amundsen_application.models.data_issue import DataIssue, Priority


class DataIssueTest(unittest.TestCase):

    def setUp(self) -> None:
        self.issue_key = 'key'
        self.title = 'title'
        self.url = 'https://place'
        self.status = 'open'
        self.priority = Priority.P2
        self.maxDiff = None

    def test_mapping_priority(self) -> None:
        expected_priority_name = 'major'
        expected_priority_display_name = 'P2'
        data_issue = DataIssue(issue_key=self.issue_key,
                               title=self.title,
                               url=self.url,
                               status=self.status,
                               priority=self.priority).serialize()
        self.assertEqual(data_issue['priority_display_name'], expected_priority_display_name)
        self.assertEqual(data_issue['priority_name'], expected_priority_name)
        self.assertEqual(data_issue['issue_key'], self.issue_key)
        self.assertEqual(data_issue['title'], self.title)
        self.assertEqual(data_issue['url'], self.url)
        self.assertEqual(data_issue['status'], self.status)

    def test_mapping_priorty_missing(self) -> None:
        expected_priority_name = None  # type: ignore
        expected_priority_display_name = None  # type: ignore
        data_issue = DataIssue(issue_key=self.issue_key,
                               title=self.title,
                               url=self.url,
                               status=self.status,
                               priority=None).serialize()
        self.assertEqual(data_issue['priority_display_name'], expected_priority_display_name)
        self.assertEqual(data_issue['priority_name'], expected_priority_name)
        self.assertEqual(data_issue['issue_key'], self.issue_key)
        self.assertEqual(data_issue['title'], self.title)
        self.assertEqual(data_issue['url'], self.url)
        self.assertEqual(data_issue['status'], self.status)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest

from flask import current_app
from http import HTTPStatus


from amundsen_application import create_app
from amundsen_application.api.preview.dashboard import v0
from amundsen_application.base.base_preview import BasePreview
from amundsen_application.api.preview.dashboard.dashboard_preview.preview_factory_method import \
    DefaultPreviewMethodFactory, BasePreviewMethodFactory

from unittest.mock import MagicMock


class TestV0(unittest.TestCase):

    def setUp(self) -> None:
        self.app = create_app(config_module_class='amundsen_application.config.LocalConfig')
        self.app_context = self.app.app_context()
        self.app_context.push()
        self.app.config['CREDENTIALS_MODE_ADMIN_TOKEN'] = 'CREDENTIALS_MODE_ADMIN_TOKEN'
        self.app.config['CREDENTIALS_MODE_ADMIN_PASSWORD'] = 'CREDENTIALS_MODE_ADMIN_PASSWORD'
        self.app.config['MODE_ORGANIZATION'] = 'foo'

    def tearDown(self) -> None:
        self.app_context.pop()

    def test_app_exists(self) -> None:
        self.assertFalse(current_app is None)

    def test_v0_default_initialize_preview_factory_class(self) -> None:
        v0.initialize_preview_factory_class()
        self.assertTrue(isinstance(v0.PREVIEW_FACTORY, DefaultPreviewMethodFactory))

    def test_v0_initialize_preview_factory_class(self) -> None:
        self.app.config['DASHBOARD_PREVIEW_FACTORY'] = DummyPreviewMethodFactory()

        v0.initialize_preview_factory_class()
        self.assertTrue(isinstance(v0.PREVIEW_FACTORY, DummyPreviewMethodFactory))

    def test_get_preview_image_not_available(self) -> None:
        mock_factory = MagicMock()
        mock_factory.get_instance.return_value.get_preview_image.side_effect = FileNotFoundError('foo')

        self.app.config['DASHBOARD_PREVIEW_FACTORY'] = mock_factory
        v0.initialize_preview_factory_class()

        response = v0.get_preview_image(uri='foo')
        self.assertEqual(response.status_code, HTTPStatus.NOT_FOUND)

    def test_get_preview_image_failure(self) -> None:
        mock_factory = MagicMock()
        mock_factory.get_instance.return_value.get_preview_image.side_effect = Exception()

        self.app.config['DASHBOARD_PREVIEW_FACTORY'] = mock_factory
        v0.initialize_preview_factory_class()

        response = v0.get_preview_image(uri='foo')
        self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)


class DummyPreviewMethodFactory(BasePreviewMethodFactory):
    def get_instance(self, *, uri: str) -> BasePreview:
        pass



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0


class IssueConfigurationException(Exception):
    """
    Raised when there are missing configuration settings
    """
    pass



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import responses
import unittest
from unittest.mock import patch

from http import HTTPStatus

from amundsen_application import create_app
from amundsen_application.api.metadata.v0 import TABLE_ENDPOINT, LAST_INDEXED_ENDPOINT,\
    POPULAR_TABLES_ENDPOINT, TAGS_ENDPOINT, USER_ENDPOINT, DASHBOARD_ENDPOINT
from amundsen_application.config import MatchRuleObject

from amundsen_application.tests.test_utils import TEST_USER_ID

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')


class MetadataTest(unittest.TestCase):
    def setUp(self) -> None:
        self.mock_popular_tables = {
            'popular_tables': [
                {
                    'cluster': 'test_cluster',
                    'database': 'test_db',
                    'schema': 'test_schema',
                    'description': 'This is a test',
                    'name': 'test_table',
                    'key': 'test_db://test_cluster.test_schema/test_table',
                }
            ]
        }
        self.expected_parsed_popular_tables = [
            {
                'cluster': 'test_cluster',
                'database': 'test_db',
                'description': 'This is a test',
                'schema': 'test_schema',
                'type': 'table',
                'name': 'test_table',
                'key': 'test_db://test_cluster.test_schema/test_table',
                'last_updated_timestamp': None,
            }
        ]
        self.mock_metadata = {
            'cluster': 'test_cluster',
            'columns': [
                {
                    'name': 'column_1',
                    'description': 'This is a test',
                    'col_type': 'bigint',
                    'sort_order': 0,
                    'stats': [
                        {'stat_type': 'count', 'stat_val': '100', 'start_epoch': 1538352000, 'end_epoch': 1538352000},
                        {'stat_type': 'count_null', 'stat_val': '0', 'start_epoch': 1538352000, 'end_epoch': 1538352000}
                    ]
                }
            ],
            'database': 'test_db',
            'is_view': False,
            'key': 'test_db://test_cluster.test_schema/test_table',
            'last_updated_timestamp': 1563872712,
            'owners': [],
            'schema': 'test_schema',
            'name': 'test_table',
            'description': 'This is a test',
            'resource_reports': [],
            'programmatic_descriptions': [
                {'source': 'c_1', 'text': 'description c'},
                {'source': 'a_1', 'text': 'description a'},
                {'source': 'b_1', 'text': 'description b'}
            ],
            'tags': [],
            'table_readers': [
                {'user': {'email': 'test@test.com', 'first_name': None, 'last_name': None}, 'read_count': 100}
            ],
            'watermarks': [
                {'watermark_type': 'low_watermark', 'partition_key': 'ds', 'partition_value': '', 'create_time': ''},
                {'watermark_type': 'high_watermark', 'partition_key': 'ds', 'partition_value': '', 'create_time': ''}
            ],
            'table_writer': {
                'application_url': 'https://test-test.test.test',
                'name': 'test_name',
                'id': 'test_id',
                'description': 'This is a test'
            }
        }
        self.expected_parsed_metadata = {
            'badges': [],
            'cluster': 'test_cluster',
            'database': 'test_db',
            'schema': 'test_schema',
            'name': 'test_table',
            'key': 'test_db://test_cluster.test_schema/test_table',
            'description': 'This is a test',
            'tags': [],
            'table_readers': [
                {
                    'user': {
                        'email': 'test@test.com',
                        'first_name': None,
                        'last_name': None,
                        'display_name': 'test@test.com',
                        'profile_url': ''
                    },
                    'read_count': 100
                }
            ],
            'partition': {
                'is_partitioned': True,
                'key': 'ds',
                'value': '01-30-2019'
            },
            'owners': [],
            'is_view': False,
            'columns': [
                {
                    'name': 'column_1',
                    'description': 'This is a test',
                    'col_type': 'bigint',
                    'sort_order': 0,
                    'stats': [
                        {'stat_type': 'count', 'stat_val': '100', 'start_epoch': 1538352000, 'end_epoch': 1538352000},
                        {'stat_type': 'count_null', 'stat_val': '0', 'start_epoch': 1538352000, 'end_epoch': 1538352000}
                    ],
                    'is_editable': True
                }
            ],
            "programmatic_descriptions": [
                {
                    'source': 'a',
                    'text': 'description a'
                },
                {
                    'source': 'b',
                    'text': 'description b'
                },
                {
                    'source': 'c',
                    'text': 'description c'
                },
            ],
            'resource_reports': [],
            'table_writer': {
                'application_url': 'https://test-test.test.test',
                'name': 'test_name',
                'id': 'test_id',
                'description': 'This is a test'
            },
            'watermarks': [
                {'watermark_type': 'low_watermark', 'partition_key': 'ds', 'partition_value': '', 'create_time': ''},
                {'watermark_type': 'high_watermark', 'partition_key': 'ds', 'partition_value': '', 'create_time': ''}
            ],
            'source': '/source',
            'is_editable': True,
            'last_updated_timestamp': None
        }
        self.expected_related_dashboard_response = {
            "dashboards": [{
                "group_name": "Group Test One",
                "description": None,
                "cluster": "gold",
                "group_url": "https://app.mode.com/companyName/spaces/123",
                "uri": "mode_dashboard://gold.123/234rt78",
                "last_successful_run_timestamp": 1590505846,
                "name": "Test Dashboard One",
                "product": "mode",
                "url": "https://app.mode.com/companyName/reports/234rt78"
            }, {
                "group_name": "Group Test Two",
                "description": None,
                "cluster": "gold",
                "group_url": "https://app.mode.com/companyName/spaces/334",
                "uri": "mode_dashboard://gold.334/34thyu56",
                "last_successful_run_timestamp": 1590519704,
                "name": "Test Dashboard Two",
                "product": "mode",
                "url": "https://app.mode.com/companyName/reports/34thyu56"
            }, {
                "group_name": "Group Test Three",
                "description": None,
                "cluster": "gold",
                "group_url": "https://app.mode.com/companyName/spaces/789",
                "uri": "mode_dashboard://gold.789/12ed34",
                "last_successful_run_timestamp": 1590538191,
                "name": "Test Dashboard Three",
                "product": "mode",
                "url": "https://app.mode.com/companyName/reports/12ed34"
            }]
        }

        self.expected_programmatic_descriptions_with_config = {
            "programmatic_descriptions": [
                {
                    'source': 'a',
                    'text': 'description a'
                },
                {
                    'source': 'b',
                    'text': 'description b'
                },
                {
                    'source': 'c',
                    'text': 'description c'
                },
            ]
        }
        self.mock_tags = {
            'tag_usages': [
                {
                    'tag_count': 3,
                    'tag_name': 'tag_0'
                }, {
                    'tag_count': 4,
                    'tag_name': 'tag_1'
                }, {
                    'tag_count': 5,
                    'tag_name': 'tag_2'
                }, {
                    'tag_count': 10,
                    'tag_name': 'tag_3'
                }, {
                    'tag_count': 1,
                    'tag_name': 'tag_4'
                }
            ]
        }
        self.expected_parsed_tags = [
            {
                'tag_count': 3,
                'tag_name': 'tag_0'
            },
            {
                'tag_count': 4,
                'tag_name': 'tag_1'
            },
            {
                'tag_count': 5,
                'tag_name': 'tag_2'
            },
            {
                'tag_count': 10,
                'tag_name': 'tag_3'
            },
            {
                'tag_count': 1,
                'tag_name': 'tag_4'
            }
        ]
        self.mock_user = {
            'email': 'test@test.com',
            'employee_type': 'FTE',
            'first_name': 'Firstname',
            'full_name': 'Firstname Lastname',
            'github_username': 'githubusername',
            'is_active': True,
            'last_name': 'Lastname',
            'manager_id': 'managerid',
            'manager_fullname': 'Manager Fullname',
            'role_name': 'SWE',
            'slack_id': 'slackuserid',
            'team_name': 'Amundsen',
            'user_id': 'testuserid',
        }
        self.expected_parsed_user = {
            'display_name': 'Firstname Lastname',
            'email': 'test@test.com',
            'employee_type': 'FTE',
            'first_name': 'Firstname',
            'full_name': 'Firstname Lastname',
            'github_username': 'githubusername',
            'is_active': True,
            'last_name': 'Lastname',
            'manager_email': None,
            'manager_fullname': 'Manager Fullname',
            'manager_id': 'managerid',
            'profile_url': '',
            'role_name': 'SWE',
            'slack_id': 'slackuserid',
            'team_name': 'Amundsen',
            'user_id': 'testuserid',
        }
        self.get_user_resource_response = {
            'table': [
                {
                    'cluster': 'cluster',
                    'database': 'database',
                    'schema': 'schema',
                    'name': 'table_name_0',
                    'description': 'description',
                },
                {
                    'cluster': 'cluster',
                    'database': 'database',
                    'schema': 'schema',
                    'name': 'table_name_1',
                    'description': 'description',
                },
            ],
            'dashboard': [],
        }
        self.expected_parsed_user_resources = {
            'table': [
                {
                    'cluster': 'cluster',
                    'database': 'database',
                    'description': 'description',
                    'last_updated_timestamp': None,
                    'name': 'table_name_0',
                    'schema': 'schema',
                    'type': 'table',
                    'key': 'database://cluster.schema/table_name_0',
                },
                {
                    'cluster': 'cluster',
                    'database': 'database',
                    'description': 'description',
                    'last_updated_timestamp': None,
                    'name': 'table_name_1',
                    'schema': 'schema',
                    'type': 'table',
                    'key': 'database://cluster.schema/table_name_1',
                },
            ],
            'dashboard': [],
        }
        self.mock_dashboard_metadata = {
            "badges": [],
            "chart_names": [],
            "cluster": "gold",
            "created_timestamp": 1558035206,
            "description": "test description",
            "frequent_users": [],
            "group_name": "test group name",
            "group_url": "test url",
            "last_run_state": "failed",
            "last_run_timestamp": 1587395014,
            "last_successful_run_timestamp": 1578434241,
            "name": "test report name",
            "owners": [
                {
                    "display_name": "First Last",
                    "email": "email@mail.com",
                    "employee_type": "teamMember",
                    "first_name": "First",
                    "full_name": "First Last",
                    "github_username": "",
                    "is_active": True,
                    "last_name": "Last",
                    "manager_email": "",
                    "manager_fullname": "",
                    "profile_url": "",
                    "role_name": "Test Role",
                    "slack_id": "",
                    "team_name": "Team Name",
                    "user_id": "test_user_id"
                }
            ],
            "product": "mode",
            "query_names": [
                "test query 1",
                "test query 2",
                "test query 3",
            ],
            "recent_view_count": 8,
            "tables": [
                {
                    "cluster": "cluster",
                    "database": "database",
                    "description": "test description",
                    "key": "database://cluster.schema/name",
                    "last_updated_timestamp": None,
                    "name": "name",
                    "schema": "schema",
                    "type": "table"
                },
                {
                    "cluster": "cluster",
                    "database": "database",
                    "description": "test description",
                    "key": "database://cluster.schema/name_2",
                    "last_updated_timestamp": None,
                    "name": "name_2",
                    "schema": "schema",
                    "type": "table"
                },
            ],
            "tags": [
                {
                    "tag_name": "amundsen",
                    "tag_type": "default"
                },
            ],
            "updated_timestamp": 1578433917,
            "uri": "test_dashboard_uri",
            "url": "test_dashboard_url"
        }

        self.expected_parsed_dashboard = {
            "badges": [],
            "chart_names": [],
            "cluster": "gold",
            "created_timestamp": 1558035206,
            "description": "test description",
            "frequent_users": [],
            "group_name": "test group name",
            "group_url": "test url",
            "last_run_state": "failed",
            "last_run_timestamp": 1587395014,
            "last_successful_run_timestamp": 1578434241,
            "name": "test report name",
            "owners": [
                {
                    "display_name": "First Last",
                    "email": "email@mail.com",
                    "employee_type": "teamMember",
                    "first_name": "First",
                    "full_name": "First Last",
                    "github_username": "",
                    "is_active": True,
                    "last_name": "Last",
                    "manager_email": "",
                    "manager_fullname": "",
                    "profile_url": "",
                    "role_name": "Test Role",
                    "slack_id": "",
                    "team_name": "Team Name",
                    "user_id": "test_user_id"
                }
            ],
            "product": "mode",
            "query_names": [
                "test query 1",
                "test query 2",
                "test query 3",
            ],
            "recent_view_count": 8,
            "tables": [
                {
                    "cluster": "cluster",
                    "database": "database",
                    "description": "test description",
                    "key": "database://cluster.schema/name",
                    "last_updated_timestamp": None,
                    "name": "name",
                    "schema": "schema",
                    "type": "table"
                },
                {
                    "cluster": "cluster",
                    "database": "database",
                    "description": "test description",
                    "key": "database://cluster.schema/name_2",
                    "last_updated_timestamp": None,
                    "name": "name_2",
                    "schema": "schema",
                    "type": "table"
                },
            ],
            "tags": [
                {
                    "tag_name": "amundsen",
                    "tag_type": "default"
                },
            ],
            "updated_timestamp": 1578433917,
            "uri": "test_dashboard_uri",
            "url": "test_dashboard_url"
        }

    @responses.activate
    def test_popular_tables_success(self) -> None:
        """
        Test successful popular_tables request
        :return:
        """
        mock_url = local_app.config['METADATASERVICE_BASE'] \
            + POPULAR_TABLES_ENDPOINT \
            + f'/{TEST_USER_ID}'
        responses.add(responses.GET, mock_url,
                      json=self.mock_popular_tables, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/popular_tables')
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertCountEqual(data.get('results'), self.expected_parsed_popular_tables)

    @responses.activate
    def test_popular_tables_propagate_failure(self) -> None:
        """
        Test that any error codes from the request are propagated through, to be
        returned to the React application
        :return:
        """
        mock_url = local_app.config['METADATASERVICE_BASE'] \
            + POPULAR_TABLES_ENDPOINT \
            + f'/{TEST_USER_ID}'
        responses.add(responses.GET, mock_url,
                      json=self.mock_popular_tables, status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/popular_tables')
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)

    @responses.activate
    def test_popular_tables_catch_exception(self) -> None:
        """
        Test catching exception if there is an issue processing the popular table
        results from the metadata service
        :return:
        """
        mock_url = local_app.config['METADATASERVICE_BASE'] \
            + POPULAR_TABLES_ENDPOINT \
            + f'/{TEST_USER_ID}'
        responses.add(responses.GET, mock_url,
                      json={'popular_tables': None}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/popular_tables')
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    @responses.activate
    def test_get_table_metadata_success(self) -> None:
        """
        Test successful get_table_metadata request
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/db://cluster.schema/table'
        responses.add(responses.GET, url, json=self.mock_metadata, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/table',
                query_string=dict(
                    key='db://cluster.schema/table',
                    index='0',
                    source='test_source'
                )
            )
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertCountEqual(data.get('tableData'), self.expected_parsed_metadata)

    @responses.activate
    def test_update_table_owner_success(self) -> None:
        """
        Test successful update_table_owner request
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/db://cluster.schema/table/owner/test'
        responses.add(responses.PUT, url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.put(
                '/api/metadata/v0/update_table_owner',
                json={
                    'key': 'db://cluster.schema/table',
                    'owner': 'test'
                }
            )
            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_update_table_owner_propagate_failure(self) -> None:
        """
        Test that any error codes from the update_table_owner request are propagated
        to be returned to the React application
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/db://cluster.schema/table/owner/test'
        responses.add(responses.PUT, url, json={}, status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.put(
                '/api/metadata/v0/update_table_owner',
                json={
                    'key': 'db://cluster.schema/table',
                    'owner': 'test'
                }
            )
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)

    @responses.activate
    def test_get_last_indexed_success(self) -> None:
        """
        Test successful get_last_indexed request
        :return:
        """
        responses.add(responses.GET, local_app.config['METADATASERVICE_BASE'] + LAST_INDEXED_ENDPOINT,
                      json={'neo4j_latest_timestamp': 1538352000}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/get_last_indexed')
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertEqual(data.get('timestamp'), 1538352000)

    @responses.activate
    def test_get_last_indexed_propagate_failure(self) -> None:
        """
        Test that any error codes from the get_last_indexed request are propagated through,
        to be returned to the React application
        :return:
        """
        responses.add(responses.GET, local_app.config['METADATASERVICE_BASE'] + LAST_INDEXED_ENDPOINT,
                      json=None, status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/get_last_indexed')
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)

    @responses.activate
    def test_get_table_description_success(self) -> None:
        """
        Test successful get_table_description request
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/db://cluster.schema/table/description'
        responses.add(responses.GET, url, json={'description': 'This is a test'}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/get_table_description',
                query_string=dict(key='db://cluster.schema/table')
            )
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertEqual(data.get('description'), 'This is a test')

    @responses.activate
    def test_get_table_description_propagate_failure(self) -> None:
        """
        Test that any error codes from the get_table_description request are propagated through,
        to be returned to the React application
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/db://cluster.schema/table/description'
        responses.add(responses.GET, url, json={'description': 'This is a test'}, status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/get_table_description',
                query_string=dict(key='db://cluster.schema/table')
            )
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)

    @responses.activate
    def test_put_table_description_success(self) -> None:
        """
        Test successful put_table_description request
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/db://cluster.schema/table/description'
        responses.add(responses.PUT, url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.put(
                '/api/metadata/v0/put_table_description',
                json={
                    'key': 'db://cluster.schema/table',
                    'description': 'test',
                    'source': 'source'
                }
            )
            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_put_table_description_denied(self) -> None:
        """
        Test put_table_description that should be rejected due to permissions.
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + \
            '/db://cluster.schema/table/column/colA/description'
        responses.add(responses.GET, url, json={'description': 'This is a test'}, status=HTTPStatus.OK)

        with patch.dict(local_app.config, {'UNEDITABLE_SCHEMAS': set(['schema'])}):
            with local_app.test_client() as test:
                response = test.put(
                    '/api/metadata/v0/put_table_description',
                    json={
                        'key': 'db://cluster.schema/table',
                        'description': 'test',
                        'source': 'source'
                    }
                )
                self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)

    @responses.activate
    def test_get_column_description_success(self) -> None:
        """
        Test successful get_column_description request
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + \
            '/db://cluster.schema/table/column/colA/description'
        responses.add(responses.GET, url, json={'description': 'This is a test'}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/get_column_description',
                query_string=dict(
                    key='db://cluster.schema/table',
                    index='0',
                    column_name='colA'
                )
            )
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertEqual(data.get('description'), 'This is a test')

    @responses.activate
    def test_get_column_description_propagate_failure(self) -> None:
        """
        Test that any error codes from the get_column_description request are propagated through,
        to be returned to the React application
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + \
            '/db://cluster.schema/table/column/colA/description'
        responses.add(responses.GET, url, json={'description': 'This is a test'}, status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/get_column_description',
                query_string=dict(
                    key='db://cluster.schema/table',
                    index='0',
                    column_name='colA'
                )
            )
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)

    @responses.activate
    def test_put_column_description_success(self) -> None:
        """
        Test successful put_column_description request
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + \
            '/db://cluster.schema/table/column/col/description'
        responses.add(responses.PUT, url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.put(
                '/api/metadata/v0/put_column_description',
                json={
                    'key': 'db://cluster.schema/table',
                    'column_name': 'col',
                    'description': 'test',
                    'source': 'source'
                }
            )
            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_put_column_description_denied(self) -> None:
        """
        Test put_column_description on an unwritable table.
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + \
            '/db://cluster.schema/an_uneditable_table/column/col/description'
        responses.add(responses.PUT, url, json={}, status=HTTPStatus.OK)

        rule = MatchRuleObject(table_name_regex=r".*uneditable_table.*")
        with patch.dict(local_app.config, {'UNEDITABLE_TABLE_DESCRIPTION_MATCH_RULES': [rule]}):
            with local_app.test_client() as test:
                response = test.put(
                    '/api/metadata/v0/put_column_description',
                    json={
                        'key': 'db://cluster.schema/an_uneditable_table',
                        'column_name': 'col',
                        'description': 'test',
                        'source': 'source'
                    }
                )
                self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)

    @responses.activate
    def test_get_tags(self) -> None:
        """
        Test successful fetch of all tags
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TAGS_ENDPOINT
        responses.add(responses.GET, url, json=self.mock_tags, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/tags')
            data = json.loads(response.data)
            self.assertCountEqual(data.get('tags'), self.expected_parsed_tags)

    @responses.activate
    def test_update_table_tags_put(self) -> None:
        """
        Test adding a tag on a table
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/db://cluster.schema/table/tag/tag_5'
        responses.add(responses.PUT, url, json={}, status=HTTPStatus.OK)

        searchservice_base = local_app.config['SEARCHSERVICE_BASE']
        get_table_url = f'{searchservice_base}/search_table'
        responses.add(responses.POST, get_table_url,
                      json={'results': [{'id': '1', 'tags': [{'tag_name': 'tag_1'}, {'tag_name': 'tag_2'}]}]},
                      status=HTTPStatus.OK)

        post_table_url = f'{searchservice_base}/document_table'
        responses.add(responses.PUT, post_table_url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.put(
                '/api/metadata/v0/update_table_tags',
                json={
                    'key': 'db://cluster.schema/table',
                    'tag': 'tag_5'
                }
            )
            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_update_table_tags_delete(self) -> None:
        """
        Test deleting a tag on a table
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/db://cluster.schema/table/tag/tag_5'
        responses.add(responses.DELETE, url, json={}, status=HTTPStatus.OK)

        searchservice_base = local_app.config['SEARCHSERVICE_BASE']
        get_table_url = f'{searchservice_base}/search_table'
        responses.add(responses.POST, get_table_url,
                      json={'results': [{'id': '1', 'tags': [{'tag_name': 'tag_1'}, {'tag_name': 'tag_2'}]}]},
                      status=HTTPStatus.OK)

        post_table_url = f'{searchservice_base}/document_table'
        responses.add(responses.PUT, post_table_url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.delete(
                '/api/metadata/v0/update_table_tags',
                json={
                    'key': 'db://cluster.schema/table',
                    'tag': 'tag_5'
                }
            )
            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_update_dashboard_tags_put(self) -> None:
        """
        Test adding a tag on a dashboard
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + DASHBOARD_ENDPOINT + '/test_dashboard_uri/tag/test_tag'
        responses.add(responses.PUT, url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.put(
                '/api/metadata/v0/update_dashboard_tags',
                json={
                    'key': 'test_dashboard_uri',
                    'tag': 'test_tag'
                }
            )
            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_update_dashboard_tags_delete(self) -> None:
        """
        Test deleting a tag on a dashboard
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + DASHBOARD_ENDPOINT + '/test_dashboard_uri/tag/test_tag'
        responses.add(responses.DELETE, url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.delete(
                '/api/metadata/v0/update_dashboard_tags',
                json={
                    'key': 'test_dashboard_uri',
                    'tag': 'test_tag'
                }
            )
            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_get_user_failure(self) -> None:
        """
        Test get_user fails when no user_id is specified
        """
        url = local_app.config['METADATASERVICE_BASE'] + USER_ENDPOINT + '/testuser'
        responses.add(responses.GET, url, json=self.mock_user, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/user')
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    @responses.activate
    def test_get_user_success(self) -> None:
        """
        Test get_user success
        """
        url = local_app.config['METADATASERVICE_BASE'] + USER_ENDPOINT + '/testuser'
        responses.add(responses.GET, url, json=self.mock_user, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/user', query_string=dict(user_id='testuser'))
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertEqual(dict(data.get('user'), **self.expected_parsed_user), data.get('user'))

    @responses.activate
    def test_get_bookmark(self) -> None:
        """
        Test get_bookmark with no user specified
        """
        url = '{0}{1}/{2}/follow/'.format(local_app.config['METADATASERVICE_BASE'], USER_ENDPOINT, TEST_USER_ID)
        responses.add(responses.GET, url, json=self.get_user_resource_response, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/user/bookmark')
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertCountEqual(data.get('bookmarks'), self.expected_parsed_user_resources)

    @responses.activate
    def test_get_bookmark_failure(self) -> None:
        """
        Test correct response returned when get_bookmark fails
        """
        url = f"{local_app.config['METADATASERVICE_BASE']}{USER_ENDPOINT}/{TEST_USER_ID}/follow/"
        responses.add(responses.GET, url, json=self.mock_user, status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/user/bookmark')
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)
            expected = {
                'bookmarks': {'table': [], 'dashboard': []},
                'msg': 'Encountered error: failed to get bookmark for user_id: test_user_id',
            }
            self.assertEqual(response.json, expected)

    @responses.activate
    def test_get_bookmark_for_user(self) -> None:
        """
        Test get_bookmark with a specified user
        """
        specified_user = 'other_user'
        url = '{0}{1}/{2}/follow/'.format(local_app.config['METADATASERVICE_BASE'], USER_ENDPOINT, specified_user)
        responses.add(responses.GET, url, json=self.get_user_resource_response, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/user/bookmark', query_string=dict(user_id=specified_user))
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertCountEqual(data.get('bookmarks'), self.expected_parsed_user_resources)

    @responses.activate
    def test_put_bookmark(self) -> None:
        """
        Test update_bookmark with a PUT request
        """
        resource_type = 'table'
        key = 'database://cluster.schema/table_name_1'
        url = '{0}{1}/{2}/follow/{3}/{4}'.format(local_app.config['METADATASERVICE_BASE'],
                                                 USER_ENDPOINT,
                                                 TEST_USER_ID,
                                                 resource_type,
                                                 key)
        responses.add(responses.PUT, url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.put(
                '/api/metadata/v0/user/bookmark',
                json={
                    'type': resource_type,
                    'key': key,
                })

            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_delete_bookmark(self) -> None:
        """
        Test update_bookmark with a DELETE request
        """
        resource_type = 'table'
        key = 'database://cluster.schema/table_name_1'
        url = '{0}{1}/{2}/follow/{3}/{4}'.format(local_app.config['METADATASERVICE_BASE'],
                                                 USER_ENDPOINT,
                                                 TEST_USER_ID,
                                                 resource_type,
                                                 key)
        responses.add(responses.DELETE, url, json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.delete(
                '/api/metadata/v0/user/bookmark',
                json={
                    'type': resource_type,
                    'key': key,
                })

            self.assertEqual(response.status_code, HTTPStatus.OK)

    @responses.activate
    def test_get_user_read(self) -> None:
        """
        Test get_user_read API request
        """
        test_user = 'test_user'
        url = '{0}{1}/{2}/read/'.format(local_app.config['METADATASERVICE_BASE'], USER_ENDPOINT, test_user)
        responses.add(responses.GET, url, json=self.get_user_resource_response, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/user/read', query_string=dict(user_id=test_user))
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertCountEqual(data.get('read'), self.expected_parsed_user_resources.get('table'))

    @responses.activate
    def test_get_user_own(self) -> None:
        """
        Test get_user_own API request
        """
        test_user = 'test_user'
        url = '{0}{1}/{2}/own/'.format(local_app.config['METADATASERVICE_BASE'], USER_ENDPOINT, test_user)
        responses.add(responses.GET, url, json=self.get_user_resource_response, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get('/api/metadata/v0/user/own', query_string=dict(user_id=test_user))
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertCountEqual(data.get('own'), self.expected_parsed_user_resources)

    @responses.activate
    def test_get_dashboard_success(self) -> None:
        """
        Test get_dashboard API success
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + DASHBOARD_ENDPOINT + '/test_dashboard_uri'
        responses.add(responses.GET, url, json=self.mock_dashboard_metadata, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/dashboard',
                query_string=dict(
                    uri='test_dashboard_uri',
                    index='0',
                    source='test_source'
                )
            )
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertCountEqual(data.get('dashboard'), self.expected_parsed_dashboard)

    @responses.activate
    def test_get_dashboard_bad_parameters(self) -> None:
        """
        Test get_dashboard API failure with missing URI parameter
        :return:
        """
        url = local_app.config['METADATASERVICE_BASE'] + DASHBOARD_ENDPOINT + '/test_dashboard_uri'
        responses.add(responses.GET, url, json=self.mock_dashboard_metadata, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/dashboard',
                query_string=dict()
            )
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)
            self.assertCountEqual(data.get('dashboard'), {})

    @responses.activate
    def test_get_related_dashboards_success(self) -> None:
        """
        Test get_related_dashboards API success
        :return:
        """
        test_table = 'db://cluster.schema/table'
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/' + test_table + '/dashboard/'
        responses.add(responses.GET, url, json=self.expected_related_dashboard_response, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/table/{0}/dashboards'.format(test_table)
            )
            data = json.loads(response.data)

            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertEqual(
                len(data.get('dashboards')),
                len(self.expected_related_dashboard_response.get('dashboards'))
            )

    @responses.activate
    def test_get_related_dashboards_failure(self) -> None:
        """
        Test get_related_dashboards API failure
        :return:
        """
        test_table = 'db://cluster.schema/table'
        url = local_app.config['METADATASERVICE_BASE'] + TABLE_ENDPOINT + '/' + test_table + '/dashboard/'
        responses.add(responses.GET, url, json=self.expected_related_dashboard_response, status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.get(
                '/api/metadata/v0/table/{0}/dashboards'.format(test_table)
            )
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)
            expected = {
                'dashboards': [],
                'msg': 'Encountered 400 Error: Related dashboard metadata request failed',
                'status_code': 400
            }
            self.assertEqual(response.json, expected)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from unittest.mock import Mock

import flask
import unittest
from amundsen_application.proxy.issue_tracker_clients.issue_exceptions import IssueConfigurationException
from amundsen_application.proxy.issue_tracker_clients.asana_client import AsanaClient
from amundsen_application.models.data_issue import DataIssue, Priority

app = flask.Flask(__name__)
app.config.from_object('amundsen_application.config.TestConfig')


class AsanaClientTest(unittest.TestCase):

    def setUp(self) -> None:
        self.mock_issue_instance = DataIssue(issue_key='key',
                                             title='some title',
                                             url='http://somewhere',
                                             status='open',
                                             priority=Priority.P2)

    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.asana_client.asana.Client')
    def test_create_AsanaClient_validates_config(self, mock_asana_client: Mock) -> None:
        with app.test_request_context():
            try:
                AsanaClient(
                    issue_labels=[],
                    issue_tracker_url='',
                    issue_tracker_user='',
                    issue_tracker_password='',
                    issue_tracker_project_id=-1,
                    issue_tracker_max_results=-1)

            except IssueConfigurationException as e:
                self.assertTrue(type(e), type(IssueConfigurationException))
                self.assertTrue(e, 'The following config settings must be set for Asana: '
                                   'ISSUE_TRACKER_URL, ISSUE_TRACKER_USER, ISSUE_TRACKER_PASSWORD, '
                                   'ISSUE_TRACKER_PROJECT_ID')



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest

from amundsen_application.api.utils.search_utils import generate_query_json, has_filters, transform_filters


class SearchUtilsTest(unittest.TestCase):
    def setUp(self) -> None:
        self.test_filters = {
            'invalid_option': 'invalid',
            'column': 'column_name',
            'database': {
                'db1': True,
                'db2': False,
                'db3': True,
            },
            'schema': 'schema_name',
            'table': 'table_name',
            'tag': 'tag_name',
        }
        self.expected_transformed_filters = {
            'column': ['column_name'],
            'database': ['db1', 'db3'],
            'schema': ['schema_name'],
            'table': ['table_name'],
            'tag': ['tag_name'],
        }
        self.test_page_index = "1"
        self.test_term = 'hello'

    def test_transform_filters(self) -> None:
        """
        Verify that the given filters are correctly transformed
        :return:
        """
        self.assertEqual(transform_filters(filters=self.test_filters, resource='table'),
                         self.expected_transformed_filters)

    def test_generate_query_json(self) -> None:
        """
        Verify that the returned diction correctly transforms the parameters
        :return:
        """
        query_json = generate_query_json(filters=self.expected_transformed_filters,
                                         page_index=self.test_page_index,
                                         search_term=self.test_term)
        self.assertEqual(query_json.get('page_index'), int(self.test_page_index))
        self.assertEqual(query_json.get('search_request'), {
            'type': 'AND',
            'filters': self.expected_transformed_filters
        })
        self.assertEqual(query_json.get('query_term'), self.test_term)

    def test_has_filters_return_true(self) -> None:
        """
        Returns true if called with a dictionary that has values for a valid filter category
        :return:
        """
        self.assertTrue(has_filters(filters=self.expected_transformed_filters, resource='table'))

    def test_has_filters_return_false(self) -> None:
        """
        Returns false if called with a dictionary that has no values for a valid filter category
        :return:
        """
        self.assertFalse(has_filters(filters={'fake_category': ['db1']}, resource='table'))
        self.assertFalse(has_filters(filters={'tag': []}, resource='table'))
        self.assertFalse(has_filters())



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
from typing import Dict
import responses
import unittest

from http import HTTPStatus
from unittest.mock import Mock, patch

from amundsen_application import create_app
from amundsen_application.api.search.v0 import SEARCH_DASHBOARD_ENDPOINT, SEARCH_DASHBOARD_FILTER_ENDPOINT, \
    SEARCH_TABLE_ENDPOINT, SEARCH_TABLE_FILTER_ENDPOINT, SEARCH_USER_ENDPOINT

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')

MOCK_TABLE_RESULTS = {
    'total_results': 1,
    'results': [
        {
            'cluster': 'test_cluster',
            'column_names': [
                'column_1',
                'column_2',
                'column_3'
            ],
            'database': 'test_db',
            'description': 'This is a test',
            'key': 'test_key',
            'last_updated_timestamp': 1527283287,
            'name': 'test_table',
            'schema': 'test_schema',
            'schema_description': 'test_schema_description',
            'tags': [],
            'badges': []
        }
    ]
}

MOCK_PARSED_TABLE_RESULTS = [
    {
        'type': 'table',
        'cluster': 'test_cluster',
        'database': 'test_db',
        'description': 'This is a test',
        'key': 'test_key',
        'last_updated_timestamp': 1527283287,
        'name': 'test_table',
        'schema': 'test_schema',
        'schema_description': 'test_schema_description',
        'badges': []
    }
]


class SearchTable(unittest.TestCase):
    def setUp(self) -> None:
        self.mock_table_results = MOCK_TABLE_RESULTS
        self.expected_parsed_table_results = MOCK_PARSED_TABLE_RESULTS
        self.search_service_url = local_app.config['SEARCHSERVICE_BASE'] + SEARCH_TABLE_ENDPOINT
        self.search_service_filter_url = local_app.config['SEARCHSERVICE_BASE'] + SEARCH_TABLE_FILTER_ENDPOINT
        self.fe_flask_endpoint = '/api/search/v0/table'

    def test_fail_if_term_is_none(self) -> None:
        """
        Test request failure if 'term' is not provided in the request json
        :return:
        """
        with local_app.test_client() as test:
            response = test.post(self.fe_flask_endpoint, json={'pageIndex': 0})
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_fail_if_page_index_is_none(self) -> None:
        """
        Test request failure if 'pageIndex' is not provided in the request json
        :return:
        """
        with local_app.test_client() as test:
            response = test.post(self.fe_flask_endpoint, json={'term': ''})
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    @responses.activate
    @patch('amundsen_application.api.search.v0.transform_filters')
    def test_calls_transform_filters(self, transform_filter_mock: Mock) -> None:
        """
        Test transform_filters is called with the filters from the request json
        from the request_json
        :return:
        """
        test_filters = {'schema': 'test_schema'}
        responses.add(responses.POST,
                      self.search_service_filter_url,
                      json=self.mock_table_results,
                      status=HTTPStatus.OK)

        with local_app.test_client() as test:
            test.post(self.fe_flask_endpoint,
                      json={
                          'term': 'hello',
                          'pageIndex': 1,
                          'filters': test_filters,
                          'searchType': 'test'})
            transform_filter_mock.assert_called_with(filters=test_filters, resource='table')

    @responses.activate
    @patch('amundsen_application.api.search.v0.transform_filters')
    @patch('amundsen_application.api.search.v0._search_table')
    def test_calls_search_table_log_helper(self, search_table_mock: Mock, transform_filter_mock: Mock) -> None:
        """
        Test _search_table helper method is called with correct arguments for logging
        from the request_json
        :return:
        """
        test_term = 'hello'
        test_index = 1
        test_search_type = 'test'
        mock_filters = {'schema': ['test_schema']}
        transform_filter_mock.return_value = mock_filters
        responses.add(responses.POST,
                      self.search_service_filter_url,
                      json=self.mock_table_results,
                      status=HTTPStatus.OK)

        with local_app.test_client() as test:
            test.post(self.fe_flask_endpoint,
                      json={
                          'term': test_term,
                          'pageIndex': test_index,
                          'filters': {},
                          'searchType': test_search_type})
            search_table_mock.assert_called_with(filters=mock_filters,
                                                 page_index=test_index,
                                                 search_term=test_term,
                                                 search_type=test_search_type)

    @responses.activate
    @patch('amundsen_application.api.search.v0.transform_filters')
    @patch('amundsen_application.api.search.v0.has_filters')
    @patch('amundsen_application.api.search.v0.generate_query_json')
    def test_calls_generate_query_json(self,
                                       mock_generate_query_json: Mock,
                                       has_filters_mock: Mock,
                                       transform_filter_mock: Mock
                                       ) -> None:
        """
        Test generate_query_json helper method is called with correct arguments
        from the request_json if filters exist
        :return:
        """
        test_term = 'hello'
        test_index = 1
        responses.add(responses.POST,
                      self.search_service_filter_url,
                      json=self.mock_table_results,
                      status=HTTPStatus.OK)
        has_filters_mock.return_value = True
        mock_filters = {'schema': ['test_schema']}
        transform_filter_mock.return_value = mock_filters

        with local_app.test_client() as test:
            test.post(self.fe_flask_endpoint,
                      json={'term': test_term, 'pageIndex': test_index, 'filters': {}})
            mock_generate_query_json.assert_called_with(filters=mock_filters,
                                                        page_index=test_index,
                                                        search_term=test_term)

    @responses.activate
    @patch('amundsen_application.api.search.v0.has_filters')
    @patch('amundsen_application.api.search.v0.generate_query_json')
    def test_does_not_calls_generate_query_json(self, mock_generate_query_json: Mock, has_filters_mock: Mock) -> None:
        """
        Test generate_query_json helper method is not called if filters do not exist
        :return:
        """
        test_term = 'hello'
        test_index = 1
        responses.add(responses.GET, self.search_service_url, json=self.mock_table_results, status=HTTPStatus.OK)
        has_filters_mock.return_value = False

        with local_app.test_client() as test:
            test.post(self.fe_flask_endpoint, json={'term': test_term, 'pageIndex': test_index, 'filters': {}})
            mock_generate_query_json.assert_not_called()

    @responses.activate
    def test_request_success(self) -> None:
        """
        Test that the response contains the expected data and status code on success
        :return:
        """
        test_filters = {'schema': 'test_schema'}
        test_term = 'hello'
        test_index = 1
        responses.add(responses.POST,
                      self.search_service_filter_url,
                      json=self.mock_table_results,
                      status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.post(self.fe_flask_endpoint,
                                 json={'term': test_term, 'pageIndex': test_index, 'filters': test_filters})
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)

            results = data.get('tables')
            self.assertEqual(results.get('total_results'), self.mock_table_results.get('total_results'))
            self.assertEqual(results.get('results'), self.expected_parsed_table_results)

    @responses.activate
    def test_request_fail(self) -> None:
        """
        Test that the response containes the failure status code from the search service on failure
        :return:
        """
        test_filters = {'schema': 'test_schema'}
        test_term = 'hello'
        test_index = 1
        responses.add(responses.POST, self.search_service_filter_url, json={}, status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.post(self.fe_flask_endpoint,
                                 json={'term': test_term, 'pageIndex': test_index, 'filters': test_filters})
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)
            self.assertEqual(data.get('msg'), 'Encountered error: Search request failed')


class SearchUser(unittest.TestCase):
    def setUp(self) -> None:
        self.mock_search_user_results = {
            'total_results': 1,
            'results': [
                {
                    'name': 'First Last',
                    'first_name': 'First',
                    'last_name': 'Last',
                    'team_name': 'Team Name',
                    'email': 'email@email.com',
                    'manager_email': 'manager@email.com',
                    'github_username': '',
                    'is_active': True,
                    'employee_type': 'teamMember',
                    'role_name': 'SWE',
                }
            ]
        }
        self.expected_parsed_search_user_results = [
            {
                'display_name': 'First Last',
                'email': 'email@email.com',
                'employee_type': 'teamMember',
                'first_name': 'First',
                'full_name': 'First Last',
                'github_username': '',
                'is_active': True,
                'last_name': 'Last',
                'manager_email': 'manager@email.com',
                'manager_id': None,
                'manager_fullname': None,
                'profile_url': '',
                'role_name': 'SWE',
                'slack_id': None,
                'team_name': 'Team Name',
                'type': 'user',
                'user_id': 'email@email.com'
            }
        ]
        self.bad_search_results = {
            'total_results': 1,
            'results': 'Bad results to trigger exception'
        }
        self.fe_flask_endpoint = '/api/search/v0/user'

    def test_search_user_fail_if_no_query(self) -> None:
        """
        Test request failure if 'query' is not provided in the query string
        to the search endpoint
        :return:
        """
        with local_app.test_client() as test:
            response = test.get(self.fe_flask_endpoint, query_string=dict(page_index='0'))
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_search_user_fail_if_no_page_index(self) -> None:
        """
        Test request failure if 'page_index' is not provided in the query string
        to the search endpoint
        :return:
        """
        with local_app.test_client() as test:
            response = test.get(self.fe_flask_endpoint, query_string=dict(query='test'))
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    @responses.activate
    def test_search_user_success_if_no_response_from_search(self) -> None:
        """
        Test request success
        :return:
        """
        responses.add(responses.GET, local_app.config['SEARCHSERVICE_BASE'] + SEARCH_USER_ENDPOINT,
                      json={}, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get(self.fe_flask_endpoint, query_string=dict(query='test', page_index='0'))
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)

            users = data.get('users')
            self.assertEqual(len(users.get('results')), 0)
            self.assertEqual(users.get('total_results'), 0)

    @responses.activate
    def test_search_user_success(self) -> None:
        """
        Test request success
        :return:
        """
        responses.add(responses.GET, local_app.config['SEARCHSERVICE_BASE'] + SEARCH_USER_ENDPOINT,
                      json=self.mock_search_user_results, status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.get(self.fe_flask_endpoint, query_string=dict(query='test', page_index='0'))
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)

            users = data.get('users')
            self.assertEqual(users.get('total_results'), self.mock_search_user_results.get('total_results'))

    @responses.activate
    def test_search_user_fail_on_non_200_response(self) -> None:
        """
        Test request failure if search endpoint returns non-200 http code
        :return:
        """
        responses.add(responses.GET, local_app.config['SEARCHSERVICE_BASE'] + SEARCH_USER_ENDPOINT,
                      json=self.mock_search_user_results, status=HTTPStatus.INTERNAL_SERVER_ERROR)

        with local_app.test_client() as test:
            response = test.get(self.fe_flask_endpoint, query_string=dict(query='test', page_index='0'))
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)


class SearchDashboard(unittest.TestCase):
    def setUp(self) -> None:
        self.mock_results = {
            'total_results': 2,
            'results': [
                {
                    'group_name': 'Amundsen Team',
                    'group_url': 'product_dashboard://cluster.group',
                    'name': 'Amundsen Metrics Dashboard',
                    'product': 'mode',
                    'description': 'I am a dashboard',
                    'uri': 'product_dashboard://cluster.group/name',
                    'url': 'product/name',
                    'cluster': 'cluster',
                    'last_successful_run_timestamp': 1585062593,
                    'total_usage': 1
                },
                {
                    'group_name': 'Amundsen Team',
                    'group_url': 'product_dashboard://cluster.group',
                    'name': 'Amundsen Metrics Dashboard1',
                    'product': 'mode',
                    'description': 'I am a second dashboard',
                    'uri': 'product_dashboard://cluster.group/name2',
                    'url': 'product/name',
                    'cluster': 'cluster',
                    'last_successful_run_timestamp': 1585062593,
                    'total_usage': 1
                },
            ]
        }
        self.expected_parsed_results = [
            {
                'chart_names': [],
                'cluster': 'cluster',
                'description': 'I am a dashboard',
                'group_name': 'Amundsen Team',
                'group_url': 'product_dashboard://cluster.group',
                'last_successful_run_timestamp': 1585062593,
                'name': 'Amundsen Metrics Dashboard',
                'product': 'mode',
                'type': 'dashboard',
                'uri': 'product_dashboard://cluster.group/name',
                'key': 'product_dashboard://cluster.group/name',
                'url': 'product/name'
            },
            {
                'chart_names': [],
                'cluster': 'cluster',
                'description': 'I am a second dashboard',
                'group_name': 'Amundsen Team',
                'group_url': 'product_dashboard://cluster.group',
                'last_successful_run_timestamp': 1585062593,
                'name': 'Amundsen Metrics Dashboard1',
                'product': 'mode',
                'type': 'dashboard',
                'uri': 'product_dashboard://cluster.group/name2',
                'key': 'product_dashboard://cluster.group/name2',
                'url': 'product/name'
            },
        ]
        self.search_service_url = local_app.config['SEARCHSERVICE_BASE'] + SEARCH_DASHBOARD_ENDPOINT
        self.search_service_filter_url = local_app.config['SEARCHSERVICE_BASE'] + SEARCH_DASHBOARD_FILTER_ENDPOINT
        self.fe_flask_endpoint = '/api/search/v0/dashboard'

    def test_fail_if_term_is_none(self) -> None:
        """
        Test request failure if 'query' is not provided in the query string
        :return:
        """
        with local_app.test_client() as test:
            response = test.post(self.fe_flask_endpoint, json={'pageIndex': '0'})
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_fail_if_page_index_is_none(self) -> None:
        """
        Test request failure if 'page_index' is not provided in the query string
        :return:
        """
        with local_app.test_client() as test:
            response = test.post(self.fe_flask_endpoint, json={'term': 'test'})
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    @responses.activate
    @patch('amundsen_application.api.search.v0.transform_filters')
    def test_calls_transform_filters(self, transform_filter_mock: Mock) -> None:
        """
        Test transform_filters is called with the filters from the request json
        from the request_json
        :return:
        """
        test_filters: Dict = {}
        responses.add(responses.POST,
                      self.search_service_url,
                      json=self.mock_results,
                      status=HTTPStatus.OK)

        with local_app.test_client() as test:
            test.post(self.fe_flask_endpoint,
                      json={
                          'term': 'hello',
                          'pageIndex': 1,
                          'filters': test_filters,
                          'searchType': 'test'})
            transform_filter_mock.assert_called_with(filters=test_filters, resource='dashboard')

    @responses.activate
    @patch('amundsen_application.api.search.v0._search_dashboard')
    def test_calls_search_dashboard_log_helper(self, search_dashboard_mock: Mock) -> None:
        """
        Test _search_dashboard helper method is called wwith correct arguments for logging
        from the request_json
        :return:
        """
        test_term = 'hello'
        test_index = 1
        test_search_type = 'test'
        mock_filters: Dict = {}
        responses.add(responses.GET,
                      self.search_service_url,
                      body=self.mock_results,
                      status=HTTPStatus.OK)

        with local_app.test_client() as test:
            test.post(self.fe_flask_endpoint,
                      json={
                          'term': test_term,
                          'pageIndex': test_index,
                          'filters': mock_filters,
                          'searchType': test_search_type})
            search_dashboard_mock.assert_called_with(filters=mock_filters,
                                                     page_index=test_index,
                                                     search_term=test_term,
                                                     search_type=test_search_type)

    @responses.activate
    @patch('amundsen_application.api.search.v0.transform_filters')
    @patch('amundsen_application.api.search.v0.has_filters')
    @patch('amundsen_application.api.search.v0.generate_query_json')
    def test_calls_generate_query_json(self,
                                       mock_generate_query_json: Mock,
                                       has_filters_mock: Mock,
                                       transform_filter_mock: Mock
                                       ) -> None:
        """
        Test generate_query_json helper method is called with correct arguments
        from the request_json if filters exist
        :return:
        """
        test_term = 'hello'
        test_index = 1
        responses.add(responses.POST,
                      self.search_service_filter_url,
                      json=self.mock_results,
                      status=HTTPStatus.OK)
        has_filters_mock.return_value = True
        mock_filters = {'group_name': 'test'}
        transform_filter_mock.return_value = mock_filters

        with local_app.test_client() as test:
            test.post(self.fe_flask_endpoint,
                      json={'term': test_term, 'pageIndex': test_index, 'filters': {}})
            mock_generate_query_json.assert_called_with(filters=mock_filters,
                                                        page_index=test_index,
                                                        search_term=test_term)

    @responses.activate
    @patch('amundsen_application.api.search.v0.has_filters')
    @patch('amundsen_application.api.search.v0.generate_query_json')
    def test_does_not_calls_generate_query_json(self, mock_generate_query_json: Mock, has_filters_mock: Mock) -> None:
        """
        Test generate_query_json helper method is not called if filters do not exist
        :return:
        """
        test_term = 'hello'
        test_index = 1
        responses.add(responses.GET, self.search_service_url, json=self.mock_results, status=HTTPStatus.OK)
        has_filters_mock.return_value = False

        with local_app.test_client() as test:
            test.post(self.fe_flask_endpoint, json={'term': test_term, 'pageIndex': test_index, 'filters': {}})
            mock_generate_query_json.assert_not_called()

    @responses.activate
    def test_request_success(self) -> None:
        """
        Test that the response contains the expected data and status code on success
        :return:
        """
        responses.add(responses.GET,
                      self.search_service_url,
                      json=self.mock_results,
                      status=HTTPStatus.OK)

        with local_app.test_client() as test:
            response = test.post(self.fe_flask_endpoint,
                                 json={'term': 'hello', 'pageIndex': '0'})
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)

            results = data.get('dashboards')
            print(results.get('results'))
            self.assertEqual(results.get('total_results'), self.mock_results.get('total_results'))
            self.assertEqual(results.get('results'), self.expected_parsed_results)

    @responses.activate
    def test_request_fail(self) -> None:
        """
        Test that the response containes the failure status code from the search service on failure
        :return:
        """
        responses.add(responses.GET,
                      self.search_service_url,
                      json=self.mock_results,
                      status=HTTPStatus.BAD_REQUEST)

        with local_app.test_client() as test:
            response = test.post(self.fe_flask_endpoint,
                                 json={'term': 'hello', 'pageIndex': '1'})
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)
            self.assertEqual(data.get('msg'), 'Encountered error: Search request failed')



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
import flask

from amundsen_application.base.base_preview_client import BasePreviewClient

app = flask.Flask(__name__)
app.config.from_object('amundsen_application.config.LocalConfig')


class PreviewClientTest(unittest.TestCase):
    def setUp(self) -> None:
        BasePreviewClient.__abstractmethods__ = frozenset()
        self.client = BasePreviewClient()

    def tearDown(self) -> None:
        pass

    def cover_abtract_methods(self) -> None:
        with app.test_request_context():
            try:
                self.client.get_preview_data()
            except NotImplementedError:
                self.assertTrue(True)
            else:
                self.assertTrue(False)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest

from amundsen_application.api.utils.metadata_utils import _convert_prog_descriptions, _sort_prog_descriptions, \
    _parse_editable_rule, is_table_editable, TableUri
from amundsen_application.config import MatchRuleObject
from amundsen_application import create_app

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')


class ProgrammaticDescriptionsTest(unittest.TestCase):
    def setUp(self) -> None:
        pass

    def test_convert_prog_descriptions(self) -> None:
        with local_app.app_context():
            # mock config
            test_config = {
                'RIGHT': {
                    'test3': {},
                    'test2': {'display_order': 0},
                },
                'LEFT': {
                    'test1': {'display_order': 1},
                    'test0': {'display_order': 0},
                },
                'test4': {'display_order': 0},
            }
            # test data
            test_desc = [
                {'source': 'test0', 'text': 'test'},
                {'source': 'test1', 'text': 'test'},
                {'source': 'test2', 'text': 'test'},
                {'source': 'test3', 'text': 'test'},
                {'source': 'test5', 'text': 'test'},
                {'source': 'test4', 'text': 'test'},
            ]
            # expected order based on mock
            expected_programmatic_desc = {
                'left': [
                    {'source': 'test0', 'text': 'test'},
                    {'source': 'test1', 'text': 'test'},
                ],
                'right': [
                    {'source': 'test2', 'text': 'test'},
                    {'source': 'test3', 'text': 'test'},
                ],
                'other': [
                    {'source': 'test4', 'text': 'test'},
                    {'source': 'test5', 'text': 'test'},
                ]
            }
            local_app.config['PROGRAMMATIC_DISPLAY'] = test_config

            result = _convert_prog_descriptions(test_desc)
            self.assertEqual(result.get('left'), expected_programmatic_desc.get('left'))
            self.assertEqual(result.get('right'), expected_programmatic_desc.get('right'))
            self.assertEqual(result.get('other'), expected_programmatic_desc.get('other'))

    def test_sort_prog_descriptions_returns_value_from_config(self) -> None:
        """
        Verify the method will return the display order from the programmtic description
        configuration if it exists for the given source
        :return:
        """
        with local_app.app_context():
            mock_order = 1
            mock_config = {
                "c_1": {
                    "display_order": mock_order
                }
            }
            in_config_value = {'source': 'c_1', 'text': 'I am a test'}
            self.assertEqual(_sort_prog_descriptions(mock_config, in_config_value), mock_order)

    def test_convert_prog_descriptions_without_config(self) -> None:
        with local_app.app_context():
            # mock config
            test_config = None
            # test data
            test_desc = [
                {'source': 'test4', 'text': 'test'},
                {'source': 'test5', 'text': 'test'},
            ]
            # expected order based on mock
            expected_programmatic_desc = {
                'other': [
                    {'source': 'test4', 'text': 'test'},
                    {'source': 'test5', 'text': 'test'},
                ]
            }
            local_app.config['PROGRAMMATIC_DISPLAY'] = test_config

            result = _convert_prog_descriptions(test_desc)
            self.assertEqual(len(result.get('left')), 0)
            self.assertEqual(len(result.get('right')), 0)
            self.assertEqual(result.get('other'), expected_programmatic_desc.get('other'))

    def test_sort_prog_descriptions_returns_default_value(self) -> None:
        """
        Verify the method will return the expected default value if programmtic decsription
        source is not included in teh configuration
        :return:
        """
        with local_app.app_context():
            mock_config = {
                "c_1": {
                    "display_order": 0
                }
            }
            not_in_config_value = {'source': 'test', 'text': 'I am a test'}
            self.assertEqual(_sort_prog_descriptions(mock_config, not_in_config_value), len(mock_config))


class UneditableTableDescriptionTest(unittest.TestCase):
    def setUp(self) -> None:
        pass

    def test_table_desc_match_rule_schema_only(self) -> None:
        # Mock match rule, table name and schema
        test_match_rule = MatchRuleObject(schema_regex=r"^(schema1)")

        # assert result for given schema and match rule
        self.assertEqual(_parse_editable_rule(test_match_rule, 'schema1', 'test_table'), False)
        self.assertEqual(_parse_editable_rule(test_match_rule, 'schema2', 'test_table'), True)

    def test_table_desc_match_rule_table_only(self) -> None:
        # Mock match rule, table name and schema
        test_match_rule = MatchRuleObject(table_name_regex=r"^noedit_([a-zA-Z_0-9]+)")

        # assert result for given table name and match rule
        self.assertEqual(_parse_editable_rule(test_match_rule, 'schema', 'noedit_test_table'), False)
        self.assertEqual(_parse_editable_rule(test_match_rule, 'schema', 'editable_test_table'), True)

    def test_table_desc_match_rule_schema_and_table(self) -> None:
        # Mock match rule, table name and schema
        test_match_rule = MatchRuleObject(schema_regex=r"^(schema1|schema2)",
                                          table_name_regex=r"^other_([a-zA-Z_0-9]+)")
        # assert result for given schema, table name and match rule
        self.assertEqual(_parse_editable_rule(test_match_rule, 'schema1', 'other_test_table'), False)
        self.assertEqual(_parse_editable_rule(test_match_rule, 'schema1', 'test_table'), True)
        self.assertEqual(_parse_editable_rule(test_match_rule, 'schema3', 'other_test_table'), True)
        self.assertEqual(_parse_editable_rule(test_match_rule, 'schema3', 'test_table'), True)


class TableEditabilityWrapper(unittest.TestCase):
    def setUp(self) -> None:
        pass

    def test_empty_allowed(self) -> None:
        mockConfig = {
            'UNEDITABLE_SCHEMAS': [],
            'UNEDITABLE_TABLE_DESCRIPTION_MATCH_RULES': [],
        }

        self.assertTrue(is_table_editable('anyschema', 'anytable', mockConfig))

    def test_schema(self) -> None:
        mockConfig = {
            'UNEDITABLE_SCHEMAS': ['uneditable_schema'],
            'UNEDITABLE_TABLE_DESCRIPTION_MATCH_RULES': [],
        }

        self.assertTrue(is_table_editable('different_schema', 'anytable', mockConfig))
        self.assertFalse(is_table_editable('uneditable_schema', 'anytable', mockConfig))

    def test_schema_match_rule(self) -> None:
        mockConfig = {
            'UNEDITABLE_SCHEMAS': [''],
            'UNEDITABLE_TABLE_DESCRIPTION_MATCH_RULES': [
                MatchRuleObject(schema_regex=r"^(uneditable).*"),
            ],
        }

        self.assertTrue(is_table_editable('not_uneditable_schema', 'anytable', mockConfig))
        self.assertFalse(is_table_editable('uneditable_schema', 'anytable', mockConfig))

    def test_schema_table_match_rule(self) -> None:
        mockConfig = {
            'UNEDITABLE_SCHEMAS': [''],
            'UNEDITABLE_TABLE_DESCRIPTION_MATCH_RULES': [
                MatchRuleObject(schema_regex=r"^first.*", table_name_regex=r".*bad.*")
            ],
        }

        self.assertFalse(is_table_editable('first', 'bad', mockConfig))
        self.assertFalse(is_table_editable('first', 'also_bad', mockConfig))
        self.assertTrue(is_table_editable('first', 'good', mockConfig))
        self.assertTrue(is_table_editable('not_first', 'bad', mockConfig))
        self.assertTrue(is_table_editable('second', 'bad', mockConfig))


class TableUriObject(unittest.TestCase):
    def test_simple_constructor(self) -> None:
        uri = TableUri("db", "clstr", "schm", "tbl")
        self.assertEqual(str(uri), 'db://clstr.schm/tbl')

    def test_from_uri_factory(self) -> None:
        uri = TableUri.from_uri("db://clstr.with.dots.schm/tbl/with/slashes")
        self.assertEqual(uri.database, 'db')
        self.assertEqual(uri.cluster, 'clstr.with.dots')
        self.assertEqual(uri.schema, 'schm')
        self.assertEqual(uri.table, 'tbl/with/slashes')



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest

from http import HTTPStatus
from typing import List

import flask

from amundsen_application.base.base_announcement_client import BaseAnnouncementClient
from amundsen_application.models.announcements import Announcements

app = flask.Flask(__name__)
app.config.from_object('amundsen_application.config.LocalConfig')


class MockClient(BaseAnnouncementClient):
    def __init__(self, posts: List) -> None:
        self.posts = posts
        self.announcements = Announcements(posts=posts)

    def get_posts(self) -> Announcements:
        return self.announcements


class MockExceptionClient(BaseAnnouncementClient):
    def __init__(self) -> None:
        pass

    def get_posts(self) -> Announcements:
        raise Exception('Simulate client raising exception')


class AnnouncementClientTest(unittest.TestCase):
    def test_get_posts_raise_exception(self) -> None:
        """
        Test catching any exception raised in get_posts(), which should result in
        a response with 500 error
        :return:
        """
        with app.test_request_context():
            response = MockExceptionClient()._get_posts()
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_get_posts_invalid_posts_no_date(self) -> None:
        """
        Test get_posts() failure if the client returns an object that does not match
        AnnouncementsSchema
        :return:
        """
        with app.test_request_context():
            invalid_posts = [{'test': 'This is not a valid list of posts'}]
            response = MockClient(invalid_posts)._get_posts()
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_get_posts_invalid_posts_no_title(self) -> None:
        """
        Test get_posts() failure if the client returns an object that does not match
        AnnouncementsSchema
        :return:
        """
        with app.test_request_context():
            invalid_posts = [{'date': 'December 31, 1999', 'not_a_title': 'This is still invalid'}]
            response = MockClient(invalid_posts)._get_posts()
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_get_posts_success(self) -> None:
        """
        Test get_posts() success
        :return:
        """
        with app.test_request_context():
            success_posts = [
                {
                    'title': 'Test Title',
                    'date': 'December 31, 1999',
                    'info_list': ['Test announcement'],
                }
            ]
            response = MockClient(success_posts)._get_posts()
            self.assertEqual(response.status_code, HTTPStatus.OK)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
import flask

from amundsen_application.base.base_issue_tracker_client import BaseIssueTrackerClient

app = flask.Flask(__name__)
app.config.from_object('amundsen_application.config.TestConfig')


class IssueTrackerClientTest(unittest.TestCase):
    def setUp(self) -> None:
        BaseIssueTrackerClient.__abstractmethods__ = frozenset()
        self.client = BaseIssueTrackerClient()

    def tearDown(self) -> None:
        pass

    def test_cover_get_issues(self) -> None:
        with app.test_request_context():
            try:
                self.client.get_issues(table_uri='test')
            except NotImplementedError:
                self.assertTrue(True)
            else:
                self.assertTrue(False)

    def test_cover_create_issue(self) -> None:
        with app.test_request_context():
            try:
                self.client.create_issue(table_uri='test', title='title', description='description')
            except NotImplementedError:
                self.assertTrue(True)
            else:
                self.assertTrue(False)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest

from http import HTTPStatus
from typing import Dict, List

from flask import Response, jsonify, make_response

from amundsen_application import create_app
from amundsen_application.base.base_mail_client import BaseMailClient

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')


class MockMailClient(BaseMailClient):
    def __init__(self, status_code: int, recipients: List = []) -> None:
        self.status_code = status_code

    def send_email(self,
                   sender: str = None,
                   recipients: List = [],
                   subject: str = None,
                   text: str = None,
                   html: str = None,
                   optional_data: Dict = {}) -> Response:
        return make_response(jsonify({}), self.status_code)


class MockBadClient(BaseMailClient):
    def __init__(self) -> None:
        pass

    def send_email(self,
                   sender: str = None,
                   recipients: List = [],
                   subject: str = None,
                   text: str = None,
                   html: str = None,
                   optional_data: Dict = {}) -> Response:
        raise Exception('Bad client')


class MailTest(unittest.TestCase):
    def test_feedback_client_not_implemented(self) -> None:
        """
        Test mail client is not implemented, and endpoint should return appropriate code
        :return:
        """
        with local_app.test_client() as test:
            response = test.post('/api/mail/v0/feedback', json={
                'rating': '10', 'comment': 'test'
            })
            self.assertEqual(response.status_code, HTTPStatus.NOT_IMPLEMENTED)

    def test_feedback_client_success(self) -> None:
        """
        Test mail client success
        :return:
        """
        status_codes = [HTTPStatus.OK, HTTPStatus.ACCEPTED]
        for status_code in status_codes:
            local_app.config['MAIL_CLIENT'] = MockMailClient(status_code=status_code)
            with self.subTest():
                with local_app.test_client() as test:
                    response = test.post('/api/mail/v0/feedback', json={
                        'rating': '10', 'comment': 'test'
                    })
                    self.assertTrue(200 <= response.status_code <= 300)

    def test_feedback_client_raise_exception(self) -> None:
        """
        Test failure due to incorrect implementation of base_mail_client
        :return:
        """
        local_app.config['MAIL_CLIENT'] = MockBadClient()
        with local_app.test_client() as test:
            response = test.post('/api/mail/v0/feedback', json={
                'rating': '10', 'comment': 'test'
            })
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_feedback_client_propagate_status_code(self) -> None:
        """
        Test that specific status codes returned from a custom mail client propagate,
        so that they may be appropriately logged and surfaced to the React application
        :return:
        """
        expected_code = HTTPStatus.BAD_REQUEST
        local_app.config['MAIL_CLIENT'] = MockMailClient(status_code=expected_code)
        with local_app.test_client() as test:
            response = test.post('/api/mail/v0/feedback', json={
                'rating': '10', 'comment': 'test'
            })
            self.assertEqual(response.status_code, expected_code)

    @unittest.mock.patch('amundsen_application.api.mail.v0.send_notification')
    def test_notification_endpoint_calls_send_notification(self, send_notification_mock) -> None:
        """
        Test that the endpoint calls send_notification with the correct information
        from the request json
        :return:
        """
        test_recipients = ['test@test.com']
        test_notification_type = 'added'
        test_options = {}

        with local_app.test_client() as test:
            test.post('/api/mail/v0/notification', json={
                'recipients': test_recipients,
                'notificationType': test_notification_type,
                'options': test_options,
            })
            send_notification_mock.assert_called_with(
                notification_type=test_notification_type,
                options=test_options,
                recipients=test_recipients,
                sender=local_app.config['AUTH_USER_METHOD'](local_app).email
            )

    @unittest.mock.patch('amundsen_application.api.mail.v0.send_notification')
    def test_notification_endpoint_fails_missing_notification_type(self, send_notification_mock) -> None:
        """
        Test that the endpoint fails if notificationType is not provided in the
        request json
        :return:
        """
        test_recipients = ['test@test.com']
        test_sender = 'test2@test.com'
        test_options = {}

        with local_app.test_client() as test:
            response = test.post('/api/mail/v0/notification', json={
                'recipients': test_recipients,
                'sender': test_sender,
                'options': test_options,
            })
            self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)
            self.assertFalse(send_notification_mock.called)

    @unittest.mock.patch('amundsen_application.api.mail.v0.send_notification')
    def test_notification_endpoint_fails_with_exception(self, send_notification_mock) -> None:
        """
        Test that the endpoint returns 500 exception when error occurs
        and that send_notification is not called
        :return:
        """
        with local_app.test_client() as test:
            # generates error
            response = test.post('/api/mail/v0/notification', json=None)

            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)
            self.assertFalse(send_notification_mock.called)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from enum import Enum
from typing import Optional


class Priority(Enum):
    P0 = ('P0', 'Blocker')
    P1 = ('P1', 'Critical')
    P2 = ('P2', 'Major')
    P3 = ('P3', 'Minor')

    def __init__(self, level: str, jira_severity: str):
        self.level = level
        self.jira_severity = jira_severity

    # JIRA SDK does not return priority beyond the name
    @staticmethod
    def from_jira_severity(jira_severity: str) -> 'Optional[Priority]':
        jira_severity_to_priority = {
            p.jira_severity: p for p in Priority
        }

        return jira_severity_to_priority.get(jira_severity)

    @staticmethod
    def from_level(level: str) -> 'Optional[Priority]':
        level_to_priority = {
            p.level: p for p in Priority
        }

        return level_to_priority.get(level)


class DataIssue:
    def __init__(self,
                 issue_key: str,
                 title: str,
                 url: str,
                 status: str,
                 priority: Optional[Priority]) -> None:
        self.issue_key = issue_key
        self.title = title
        self.url = url
        self.status = status
        self.priority = priority

    def serialize(self) -> dict:
        return {'issue_key': self.issue_key,
                'title': self.title,
                'url': self.url,
                'status': self.status,
                'priority_name': self.priority.jira_severity.lower() if self.priority else None,
                'priority_display_name': self.priority.level if self.priority else None}



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from http import HTTPStatus
import logging
from typing import Dict  # noqa: F401

from flask import Response, jsonify, make_response, current_app as app
from pyarrow import flight

from amundsen_application.base.base_superset_preview_client import BasePreviewClient
from amundsen_application.models.preview_data import PreviewData, PreviewDataSchema, ColumnItem


class _DremioAuthHandler(flight.ClientAuthHandler):
    """ClientAuthHandler for connections to Dremio server endpoint.
    """
    def __init__(self, username: str, password: str) -> None:
        self.username = username
        self.password = password
        super(flight.ClientAuthHandler, self).__init__()

    def authenticate(self, outgoing: flight.ClientAuthSender,
                     incoming: flight.ClientAuthReader) -> None:
        """Authenticate with Dremio user credentials.
        """
        basic_auth = flight.BasicAuth(self.username, self.password)
        outgoing.write(basic_auth.serialize())
        self.token = incoming.read()

    def get_token(self,) -> str:
        """Get the token from this AuthHandler.
        """
        return self.token


class DremioPreviewClient(BasePreviewClient):

    SQL_STATEMENT = 'SELECT * FROM {schema}."{table}" LIMIT 50'

    def __init__(self,) -> None:
        self.url = app.config['PREVIEW_CLIENT_URL']
        self.username = app.config['PREVIEW_CLIENT_USERNAME']
        self.password = app.config['PREVIEW_CLIENT_PASSWORD']

        self.connection_args: Dict[str, bytes] = {}
        tls_root_certs_path = app.config['PREVIEW_CLIENT_CERTIFICATE']
        if tls_root_certs_path is not None:
            with open(tls_root_certs_path, "rb") as f:
                self.connection_args["tls_root_certs"] = f.read()

    def get_preview_data(self, params: Dict, optionalHeaders: Dict = None) -> Response:
        """Preview data from Dremio source
        """
        database = params.get('database')
        if database != 'DREMIO':
            logging.info('Skipping table preview for non-Dremio table')
            return make_response(jsonify({'preview_data': {}}), HTTPStatus.OK)

        try:
            # Format base SQL_STATEMENT with request table and schema
            schema = '"{}"'.format(params['schema'].replace('.', '"."'))
            table = params['tableName']
            sql = DremioPreviewClient.SQL_STATEMENT.format(schema=schema,
                                                           table=table)

            client = flight.FlightClient(self.url, **self.connection_args)
            client.authenticate(_DremioAuthHandler(self.username, self.password))
            flight_descriptor = flight.FlightDescriptor.for_command(sql)
            flight_info = client.get_flight_info(flight_descriptor)
            reader = client.do_get(flight_info.endpoints[0].ticket)

            result = reader.read_all()
            names = result.schema.names
            types = result.schema.types

            columns = map(lambda x: x.to_pylist(), result.columns)
            rows = [dict(zip(names, row)) for row in zip(*columns)]
            column_items = [ColumnItem(n, t) for n, t in zip(names, types)]

            preview_data = PreviewData(column_items, rows)

            data = PreviewDataSchema().dump(preview_data)[0]
            errors = PreviewDataSchema().load(data)[1]
            if errors:
                logging.error(f'Error(s) occurred while building preview data: {errors}')
                payload = jsonify({'preview_data': {}})
                return make_response(payload, HTTPStatus.INTERNAL_SERVER_ERROR)
            else:
                payload = jsonify({'preview_data': data})
                return make_response(payload, HTTPStatus.OK)

        except Exception as e:
            logging.error(f'Encountered exception: {e}')
            payload = jsonify({'preview_data': {}})
            return make_response(payload, HTTPStatus.INTERNAL_SERVER_ERROR)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

# flake8: noqa
import unittest
from unittest.mock import patch

import requests
from requests.auth import HTTPBasicAuth
from unittest.mock import MagicMock

from amundsen_application import create_app
from amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview import ModePreview, DEFAULT_REPORT_URL_TEMPLATE
from amundsen_application.api.utils import request_utils

ACCESS_TOKEN = 'token'
PASSWORD = 'password'
ORGANIZATION = 'foo'


class TestModePreview(unittest.TestCase):

    def setUp(self) -> None:
        self.app = create_app(config_module_class='amundsen_application.config.LocalConfig')
        self.app_context = self.app.app_context()
        self.app_context.push()

    def tearDown(self) -> None:
        self.app_context.pop()

    def test_get_preview_image(self) -> None:
        with patch.object(ModePreview, '_get_preview_image_url') as mock_get_preview_image_url,\
                patch.object(requests, 'get') as mock_get:
            mock_get_preview_image_url.return_value = 'http://foo.bar/image.jpeg'
            mock_get.return_value.content = b'bar'

            preview = ModePreview(access_token='token', password='password', organization='foo')

            mode_dashboard_uri = 'mode_dashboard://gold.dg/d_id'
            actual = preview.get_preview_image(uri=mode_dashboard_uri)

            self.assertEqual(b'bar', actual)
            mock_get.assert_called_with('http://foo.bar/image.jpeg', allow_redirects=True)

    def test_get_preview_image_url_success(self) -> None:
        with patch.object(requests, 'get') as mock_get:
            mock_get.return_value.json.return_value = {'web_preview_image': 'http://foo.bar/image.jpeg'}

            preview = ModePreview(access_token='token', password='password', organization='foo')
            mode_dashboard_uri = 'mode_dashboard://gold.dg/d_id'
            url = preview._get_preview_image_url(uri=mode_dashboard_uri)

            self.assertEqual(url, 'http://foo.bar/image.jpeg')

            expected_url = DEFAULT_REPORT_URL_TEMPLATE.format(organization=ORGANIZATION, dashboard_id='d_id')
            mock_get.assert_called_with(expected_url, auth=HTTPBasicAuth(ACCESS_TOKEN, PASSWORD))

    def test_get_preview_image_url_failure_none_value(self) -> None:
        with patch.object(requests, 'get') as mock_get:
            mock_get.return_value.json.return_value = {'web_preview_image': None}

            preview = ModePreview(access_token='token', password='password', organization='foo')
            mode_dashboard_uri = 'mode_dashboard://gold.dg/d_id'
            self.assertRaises(FileNotFoundError, preview._get_preview_image_url, uri=mode_dashboard_uri)

    def test_get_preview_image_url_failure_missing_key(self) -> None:
        with patch.object(requests, 'get') as mock_get:
            mock_get.return_value.json.return_value = {}

            preview = ModePreview(access_token='token', password='password', organization='foo')
            mode_dashboard_uri = 'mode_dashboard://gold.dg/d_id'
            self.assertRaises(FileNotFoundError, preview._get_preview_image_url, uri=mode_dashboard_uri)

    def test_auth_disabled(self) -> None:
        preview = ModePreview(access_token='token', password='password', organization='foo')
        self.assertFalse(preview._is_auth_enabled)

    def test_auth_disabled_2(self) -> None:
        self.app.config['ACL_ENABLED_DASHBOARD_PREVIEW'] = {'FooPreview'}
        self.app.config['AUTH_USER_METHOD'] = MagicMock()

        preview = ModePreview(access_token='token', password='password', organization='foo')
        self.assertFalse(preview._is_auth_enabled)

    def test_auth_enabled(self) -> None:
        self.app.config['ACL_ENABLED_DASHBOARD_PREVIEW'] = {'ModePreview'}
        self.app.config['AUTH_USER_METHOD'] = MagicMock()

        preview = ModePreview(access_token='token', password='password', organization='foo')
        self.assertTrue(preview._is_auth_enabled)

    def test_authorization(self) -> None:
        self.app.config['ACL_ENABLED_DASHBOARD_PREVIEW'] = {'ModePreview'}
        self.app.config['AUTH_USER_METHOD'] = MagicMock()

        with patch('amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview.request_metadata') as mock_request_metadata:
            mock_request_metadata.return_value.json.return_value = {
                'employee_type': 'teamMember',
                'full_name': 'test_full_name',
                'is_active': 'True',
                'github_username': 'test-github',
                'slack_id': 'test_id',
                'last_name': 'test_last_name',
                'first_name': 'test_first_name',
                'team_name': 'test_team',
                'email': 'test_email',
                'other_key_values': {
                    'mode_user_id': 'foo_mode_user_id'
                }
            }

            preview = ModePreview(access_token='token', password='password', organization='foo')
            preview._authorize_access(user_id='test_email')

        with patch('amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview.request_metadata') as mock_request_metadata:
            mock_request_metadata.return_value.json.return_value = {
                'employee_type': 'teamMember',
                'full_name': 'test_full_name',
                'is_active': 'False',
                'github_username': 'test-github',
                'slack_id': 'test_id',
                'last_name': 'test_last_name',
                'first_name': 'test_first_name',
                'team_name': 'test_team',
                'email': 'test_email',
                'other_key_values': {
                    'mode_user_id': 'foo_mode_user_id'
                }
            }

            preview = ModePreview(access_token='token', password='password', organization='foo')
            self.assertRaises(PermissionError, preview._authorize_access, user_id='test_email')

        with patch('amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview.request_metadata') as mock_request_metadata:
            mock_request_metadata.return_value.json.return_value = {
                'employee_type': 'teamMember',
                'full_name': 'test_full_name',
                'is_active': 'True',
                'github_username': 'test-github',
                'slack_id': 'test_id',
                'last_name': 'test_last_name',
                'first_name': 'test_first_name',
                'team_name': 'test_team',
                'email': 'test_email',
                'other_key_values': {}
            }

            preview = ModePreview(access_token='token', password='password', organization='foo')
            self.assertRaises(PermissionError, preview._authorize_access, user_id='test_email')

        with patch('amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview.request_metadata') as mock_request_metadata:
            mock_request_metadata.return_value.json.return_value = {
                'employee_type': 'teamMember',
                'full_name': 'test_full_name',
                'is_active': 'True',
                'github_username': 'test-github',
                'slack_id': 'test_id',
                'last_name': 'test_last_name',
                'first_name': 'test_first_name',
                'team_name': 'test_team',
                'email': 'test_email',
                'other_key_values': {
                    'foo': 'bar'
                }
            }

            preview = ModePreview(access_token='token', password='password', organization='foo')
            self.assertRaises(PermissionError, preview._authorize_access, user_id='test_email')

        with patch('amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview.request_metadata') as mock_request_metadata:
            mock_request_metadata.return_value.json.return_value = {
                'employee_type': 'teamMember',
                'full_name': 'test_full_name',
                'is_active': 'True',
                'github_username': 'test-github',
                'slack_id': 'test_id',
                'last_name': 'test_last_name',
                'first_name': 'test_first_name',
                'team_name': 'test_team',
                'email': 'test_email',
                'other_key_values': None
            }

            preview = ModePreview(access_token='token', password='password', organization='foo')
            self.assertRaises(PermissionError, preview._authorize_access, user_id='test_email')

        with patch('amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview.request_metadata') as mock_request_metadata:
            mock_request_metadata.return_value.json.return_value = {
                'employee_type': 'teamMember',
                'full_name': 'test_full_name',
                'is_active': 'True',
                'github_username': 'test-github',
                'slack_id': 'test_id',
                'last_name': 'test_last_name',
                'first_name': 'test_first_name',
                'team_name': 'test_team',
                'email': 'test_email',
            }

            preview = ModePreview(access_token='token', password='password', organization='foo')
            self.assertRaises(PermissionError, preview._authorize_access, user_id='test_email')


if __name__ == '__main__':
    unittest.main()



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from typing import Dict, List  # noqa: F401


# These can move to a configuration when we have custom use cases outside of these default values
valid_search_fields = {
    'table': {
        'badges',
        'column',
        'database',
        'schema',
        'table',
        'tag'
    },
    'dashboard': {
        'group_name',
        'name',
        'product',
        'tag'
    }
}


def map_table_result(result: Dict) -> Dict:
    return {
        'type': 'table',
        'key': result.get('key', None),
        'name': result.get('name', None),
        'cluster': result.get('cluster', None),
        'description': result.get('description', None),
        'database': result.get('database', None),
        'schema': result.get('schema', None),
        'schema_description': result.get('schema_description', None),
        'badges': result.get('badges', None),
        'last_updated_timestamp': result.get('last_updated_timestamp', None),
    }


def transform_filters(*, filters: Dict = {}, resource: str) -> Dict:
    """
    Transforms the data shape of filters from the application to the data
    shape required by the search service according to the api defined at:
    https://github.com/lyft/amundsensearchlibrary/blob/master/search_service/api/swagger_doc/table/search_table_filter.yml
    https://github.com/lyft/amundsensearchlibrary/blob/master/search_service/api/swagger_doc/dashboard/search_dashboard_filter.yml
    """
    filter_payload = {}
    for category in valid_search_fields.get(resource, {}):
        values = filters.get(category)
        value_list = []  # type: List
        if values is not None:
            if type(values) == str:
                value_list = [values, ]
            elif type(values) == dict:
                value_list = [key for key in values.keys() if values[key] is True]
        if len(value_list) > 0:
            filter_payload[category] = value_list

    return filter_payload


def generate_query_json(*, filters: Dict = {}, page_index: int, search_term: str) -> Dict:
    """
    Transforms the given paramaters to the query json for the search service according to
    the api defined at:
    https://github.com/lyft/amundsensearchlibrary/blob/master/search_service/api/swagger_doc/table/search_table_filter.yml
    https://github.com/lyft/amundsensearchlibrary/blob/master/search_service/api/swagger_doc/dashboard/search_dashboard_filter.yml
    """
    return {
        'page_index': int(page_index),
        'search_request': {
            'type': 'AND',
            'filters': filters
        },
        'query_term': search_term
    }


def has_filters(*, filters: Dict = {}, resource: str = '') -> bool:
    """
    Returns whether or not the filter dictionary passed to the search service
    has at least one filter value for a valid filter category
    """
    for category in valid_search_fields.get(resource, {}):
        filter_list = filters.get(category, [])
        if len(filter_list) > 0:
            return True
    return False



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from unittest.mock import Mock

import flask
import unittest
from amundsen_application.proxy.issue_tracker_clients.issue_exceptions import IssueConfigurationException
from amundsen_application.proxy.issue_tracker_clients.jira_client import JiraClient, SEARCH_STUB_ALL_ISSUES
from amundsen_application.models.data_issue import DataIssue, Priority
from jira import JIRAError
from typing import Dict, List

app = flask.Flask(__name__)
app.config.from_object('amundsen_application.config.TestConfig')


class MockJiraResultList(list):
    def __init__(self, iterable: Dict[str, str], _total: int) -> None:
        if iterable is not None:
            list.__init__(self, iterable)
        else:
            list.__init__(self)
        self.total = _total


class JiraClientTest(unittest.TestCase):

    def setUp(self) -> None:
        self.mock_issue = {
            'issue_key': 'key',
            'title': 'some title',
            'url': 'http://somewhere',
            'status': 'open',
            'priority_name': 'Major',
            'priority_display_name': 'P2'
        }
        result_list = MockJiraResultList(iterable=self.mock_issue, _total=0)
        self.mock_jira_issues = result_list
        self.mock_issue_instance = DataIssue(issue_key='key',
                                             title='some title',
                                             url='http://somewhere',
                                             status='open',
                                             priority=Priority.P2)

    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.JIRA')
    def test_create_JiraClient_validates_config(self, mock_JIRA_client: Mock) -> None:
        with app.test_request_context():
            try:

                JiraClient(issue_labels=[],
                           issue_tracker_url='',
                           issue_tracker_user='',
                           issue_tracker_password='',
                           issue_tracker_project_id=-1,
                           issue_tracker_max_results=-1)
            except IssueConfigurationException as e:
                self.assertTrue(type(e), type(IssueConfigurationException))
                self.assertTrue(e, 'The following config settings must be set for Jira: '
                                   'ISSUE_TRACKER_URL, ISSUE_TRACKER_USER, ISSUE_TRACKER_PASSWORD, '
                                   'ISSUE_TRACKER_PROJECT_ID')

    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.JIRA')
    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.'
                         'JiraClient._generate_all_issues_url')
    def test_get_issues_returns_JIRAError(self, mock_remaining_issues: Mock, mock_JIRA_client: Mock) -> None:
        mock_JIRA_client.return_value.get_issues.side_effect = JIRAError('Some exception')
        mock_remaining_issues.return_value = 0
        with app.test_request_context():
            try:
                jira_client = JiraClient(issue_labels=[],
                                         issue_tracker_url=app.config['ISSUE_TRACKER_URL'],
                                         issue_tracker_user=app.config['ISSUE_TRACKER_USER'],
                                         issue_tracker_password=app.config['ISSUE_TRACKER_PASSWORD'],
                                         issue_tracker_project_id=app.config['ISSUE_TRACKER_PROJECT_ID'],
                                         issue_tracker_max_results=app.config['ISSUE_TRACKER_MAX_RESULTS'])
                jira_client.get_issues('key')
            except JIRAError as e:
                self.assertTrue(type(e), type(JIRAError))
                self.assertTrue(e, 'Some exception')

    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.JIRA')
    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.'
                         'JiraClient._get_issue_properties')
    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.'
                         'jira_client.JiraClient._generate_all_issues_url')
    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.'
                         'jira_client.JiraClient._sort_issues')
    def test_get_issues_returns_issues(self,
                                       mock_sort_issues: Mock,
                                       mock_get_url: Mock,
                                       mock_get_issue_properties: Mock,
                                       mock_JIRA_client: Mock) -> None:
        mock_JIRA_client.return_value.search_issues.return_value = self.mock_jira_issues
        mock_get_issue_properties.return_value = self.mock_issue
        mock_get_url.return_value = 'url'
        mock_sort_issues.return_value = [self.mock_issue]
        with app.test_request_context():
            jira_client = JiraClient(issue_labels=[],
                                     issue_tracker_url=app.config['ISSUE_TRACKER_URL'],
                                     issue_tracker_user=app.config['ISSUE_TRACKER_USER'],
                                     issue_tracker_password=app.config['ISSUE_TRACKER_PASSWORD'],
                                     issue_tracker_project_id=app.config['ISSUE_TRACKER_PROJECT_ID'],
                                     issue_tracker_max_results=app.config['ISSUE_TRACKER_MAX_RESULTS'])
            results = jira_client.get_issues(table_uri='key')
            mock_JIRA_client.assert_called
            self.assertEqual(results.issues[0], self.mock_issue)
            self.assertEqual(results.total, self.mock_jira_issues.total)
            mock_JIRA_client.return_value.search_issues.assert_called_with(
                SEARCH_STUB_ALL_ISSUES.format(table_key="key"),
                maxResults=3)

    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.JIRA')
    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.urllib.parse.quote')
    def test__generate_all_issues_url(self, mock_url_lib: Mock, mock_JIRA_client: Mock) -> None:
        mock_url_lib.return_value = 'test'
        with app.test_request_context():
            jira_client = JiraClient(issue_labels=[],
                                     issue_tracker_url=app.config['ISSUE_TRACKER_URL'],
                                     issue_tracker_user=app.config['ISSUE_TRACKER_USER'],
                                     issue_tracker_password=app.config['ISSUE_TRACKER_PASSWORD'],
                                     issue_tracker_project_id=app.config['ISSUE_TRACKER_PROJECT_ID'],
                                     issue_tracker_max_results=app.config['ISSUE_TRACKER_MAX_RESULTS'])
            issues = [DataIssue(issue_key='key', title='title', url='url', status='open', priority=Priority.P2)]
            url = jira_client._generate_all_issues_url(table_uri="table", issues=issues)
            self.assertEqual(url, 'test_url/issues/?jql=test')

    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.JIRA')
    def test__generate_all_issues_url_no_issues(self, mock_JIRA_client: Mock) -> None:
        with app.test_request_context():
            jira_client = JiraClient(issue_labels=[],
                                     issue_tracker_url=app.config['ISSUE_TRACKER_URL'],
                                     issue_tracker_user=app.config['ISSUE_TRACKER_USER'],
                                     issue_tracker_password=app.config['ISSUE_TRACKER_PASSWORD'],
                                     issue_tracker_project_id=app.config['ISSUE_TRACKER_PROJECT_ID'],
                                     issue_tracker_max_results=app.config['ISSUE_TRACKER_MAX_RESULTS'])
            issues: List[DataIssue]
            issues = []
            url = jira_client._generate_all_issues_url(table_uri="table", issues=issues)
            self.assertEqual(url, '')

    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.JIRA')
    def test_create_returns_JIRAError(self, mock_JIRA_client: Mock) -> None:
        mock_JIRA_client.return_value.create_issue.side_effect = JIRAError('Some exception')
        with app.test_request_context():
            try:
                jira_client = JiraClient(issue_labels=[],
                                         issue_tracker_url=app.config['ISSUE_TRACKER_URL'],
                                         issue_tracker_user=app.config['ISSUE_TRACKER_USER'],
                                         issue_tracker_password=app.config['ISSUE_TRACKER_PASSWORD'],
                                         issue_tracker_project_id=app.config['ISSUE_TRACKER_PROJECT_ID'],
                                         issue_tracker_max_results=app.config['ISSUE_TRACKER_MAX_RESULTS'])
                jira_client.create_issue(description='desc', table_uri='key', title='title')
            except JIRAError as e:
                self.assertTrue(type(e), type(JIRAError))
                self.assertTrue(e, 'Some exception')

    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.JIRA')
    @unittest.mock.patch('amundsen_application.proxy.issue_tracker_clients.jira_client.'
                         'JiraClient._get_issue_properties')
    def test_create_issue(self, mock_get_issue_properties: Mock, mock_JIRA_client: Mock) -> None:
        mock_JIRA_client.return_value.create_issue.return_value = self.mock_issue
        mock_get_issue_properties.return_value = self.mock_issue_instance
        mock_labels = ['mock-label']
        with app.test_request_context():
            jira_client = JiraClient(issue_labels=mock_labels,
                                     issue_tracker_url=app.config['ISSUE_TRACKER_URL'],
                                     issue_tracker_user=app.config['ISSUE_TRACKER_USER'],
                                     issue_tracker_password=app.config['ISSUE_TRACKER_PASSWORD'],
                                     issue_tracker_project_id=app.config['ISSUE_TRACKER_PROJECT_ID'],
                                     issue_tracker_max_results=app.config['ISSUE_TRACKER_MAX_RESULTS'])
            results = jira_client.create_issue(description='desc', table_uri='key', title='title')
            mock_JIRA_client.assert_called
            self.assertEqual(results, self.mock_issue_instance)
            mock_JIRA_client.return_value.create_issue.assert_called_with(fields=dict(project={
                'id': app.config["ISSUE_TRACKER_PROJECT_ID"]
            }, issuetype={
                'id': 1,
                'name': 'Bug',
            }, labels=mock_labels,
                summary='title',
                description='desc \n Reported By: test@email.com \n Table Key: key [PLEASE DO NOT REMOVE]',
                reporter={'name': 'test'}))



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from random import randint
from datetime import datetime, timedelta

from amundsen_application.models.announcements import Announcements, Post
from amundsen_application.base.base_announcement_client import BaseAnnouncementClient

try:
    from sqlalchemy import Column, Integer, String, DateTime, create_engine
    from sqlalchemy.ext.declarative import declarative_base
    from sqlalchemy.orm import sessionmaker
except ModuleNotFoundError:
    pass

Base = declarative_base()


class DBAnnouncement(Base):  # type: ignore
    __tablename__ = 'announcements'

    id = Column(Integer, primary_key=True)

    date = Column(DateTime)
    title = Column(String)
    content = Column(String)


class SQLAlchemyAnnouncementClient(BaseAnnouncementClient):
    def __init__(self) -> None:
        self._setup_mysql()

    def _setup_mysql(self) -> None:
        self.engine = create_engine('sqlite:////tmp/amundsen.db', echo=True)

        session = sessionmaker(bind=self.engine)()

        # add dummy announcements to preview
        if not self.engine.dialect.has_table(self.engine, DBAnnouncement.__tablename__):
            Base.metadata.create_all(self.engine)

            announcements = []

            dummy_announcement = """
            Lorem ipsum dolor sit amet, consectetur adipiscing elit. Donec at dapibus lorem.
            Orci varius natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus.
            Suspendisse est lectus, bibendum vitae vestibulum vitae, commodo eu tortor.
            Sed rhoncus augue eget turpis interdum, eu aliquam lectus ornare. Aenean tempus in mauris vitae viverra.
            """

            for i in range(randint(5, 9)):
                announcement = DBAnnouncement(id=i + 1,
                                              date=datetime.now() + timedelta(days=i + 1),
                                              title=f'Test announcement title {i + 1}',
                                              content=dummy_announcement)

                announcements.append(announcement)

            session.add_all(announcements)
            session.commit()

    def get_posts(self) -> Announcements:
        """
        Returns an instance of amundsen_application.models.announcements.Announcements, which should match
        amundsen_application.models.announcements.AnnouncementsSchema
        """
        session = sessionmaker(bind=self.engine)()

        posts = []

        for row in session.query(DBAnnouncement).order_by(DBAnnouncement.date.desc()):
            post = Post(title=row.title,
                        date=row.date.strftime('%b %d %Y %H:%M:%S'),
                        html_content=row.content)
            posts.append(post)

        return Announcements(posts)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging

from http import HTTPStatus

from flask import Response, jsonify, make_response, request
from flask import current_app as app
from flask.blueprints import Blueprint

from amundsen_application.api.exceptions import MailClientNotImplemented
from amundsen_application.api.utils.notification_utils import get_mail_client, send_notification
from amundsen_application.log.action_log import action_logging

LOGGER = logging.getLogger(__name__)

mail_blueprint = Blueprint('mail', __name__, url_prefix='/api/mail/v0')


@mail_blueprint.route('/feedback', methods=['POST'])
def feedback() -> Response:
    """
    Uses the instance of BaseMailClient client configured on the MAIL_CLIENT
    config variable to send an email with feedback data
    """
    try:
        mail_client = get_mail_client()
        data = request.form.to_dict()
        html_content = ''.join('<div><strong>{}:</strong><br/>{}</div><br/>'.format(k, v) for k, v in data.items())

        # action logging
        feedback_type = data.get('feedback-type')
        rating = data.get('rating')
        comment = data.get('comment')
        bug_summary = data.get('bug-summary')
        repro_steps = data.get('repro-steps')
        feature_summary = data.get('feature-summary')
        value_prop = data.get('value-prop')
        subject = data.get('subject') or data.get('feedback-type')

        _feedback(feedback_type=feedback_type,
                  rating=rating,
                  comment=comment,
                  bug_summary=bug_summary,
                  repro_steps=repro_steps,
                  feature_summary=feature_summary,
                  value_prop=value_prop,
                  subject=subject)

        options = {
            'email_type': 'feedback',
            'form_data': data
        }

        response = mail_client.send_email(html=html_content, subject=subject, optional_data=options)
        status_code = response.status_code

        if 200 <= status_code < 300:
            message = 'Success'
        else:
            message = 'Mail client failed with status code ' + str(status_code)
            logging.error(message)

        return make_response(jsonify({'msg': message}), status_code)
    except MailClientNotImplemented as e:
        message = 'Encountered exception: ' + str(e)
        logging.exception(message)
        return make_response(jsonify({'msg': message}), HTTPStatus.NOT_IMPLEMENTED)
    except Exception as e1:
        message = 'Encountered exception: ' + str(e1)
        logging.exception(message)
        return make_response(jsonify({'msg': message}), HTTPStatus.INTERNAL_SERVER_ERROR)


@action_logging
def _feedback(*,
              feedback_type: str,
              rating: str,
              comment: str,
              bug_summary: str,
              repro_steps: str,
              feature_summary: str,
              value_prop: str,
              subject: str) -> None:
    """ Logs the content of the feedback form """
    pass  # pragma: no cover


@mail_blueprint.route('/notification', methods=['POST'])
def notification() -> Response:
    """
    Uses the instance of BaseMailClient client configured on the MAIL_CLIENT
    config variable to send a notification email based on data passed from the request
    """
    try:
        data = request.get_json()

        notification_type = data.get('notificationType')
        if notification_type is None:
            message = 'Encountered exception: notificationType must be provided in the request payload'
            logging.exception(message)
            return make_response(jsonify({'msg': message}), HTTPStatus.BAD_REQUEST)

        sender = data.get('sender')
        if sender is None:
            sender = app.config['AUTH_USER_METHOD'](app).email

        options = data.get('options', {})
        recipients = data.get('recipients', [])

        return send_notification(
            notification_type=notification_type,
            options=options,
            recipients=recipients,
            sender=sender
        )
    except Exception as e:
        message = 'Encountered exception: ' + str(e)
        logging.exception(message)
        return make_response(jsonify({'msg': message}), HTTPStatus.INTERNAL_SERVER_ERROR)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
from http import HTTPStatus

from flask import Response

from amundsen_application import create_app
from amundsen_application.api.announcements import v0

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')
ANNOUNCEMENT_CLIENT_CLASS = ('amundsen_application.base.examples.'
                             'example_announcement_client.SQLAlchemyAnnouncementClient')


class AnnouncementTest(unittest.TestCase):

    def setUp(self) -> None:
        local_app.config['ANNOUNCEMENT_CLIENT_ENABLED'] = True

    def test_no_client_class(self) -> None:
        """
        Test that Not Implemented error is raised when PREVIEW_CLIENT is None
        :return:
        """
        # Reset side effects of other tests to ensure that the results are the
        # same regardless of execution order
        v0.ANNOUNCEMENT_CLIENT_CLASS = None
        v0.ANNOUNCEMENT_CLIENT_INSTANCE = None

        local_app.config['ANNOUNCEMENT_CLIENT'] = None
        with local_app.test_client() as test:
            response = test.get('/api/announcements/v0/')
            self.assertEqual(response.status_code, HTTPStatus.NOT_IMPLEMENTED)

    @unittest.mock.patch(ANNOUNCEMENT_CLIENT_CLASS + '._get_posts')
    def test_good_client_response(self, mock_get_posts) -> None:
        """
        :return:
        """
        local_app.config['ANNOUNCEMENT_CLIENT'] = ANNOUNCEMENT_CLIENT_CLASS
        mock_get_posts.return_value = Response(response='',
                                               status=HTTPStatus.OK)
        with local_app.test_client() as test:
            response = test.get('/api/announcements/v0/')
            self.assertEqual(response.status_code, HTTPStatus.OK)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import getpass
import socket
import unittest
from contextlib import contextmanager
from typing import Generator, Any

import flask

from amundsen_application.log import action_log, action_log_callback
from amundsen_application.log.action_log import action_logging
from amundsen_application.log.action_log import get_epoch_millisec

# from mock import patch
# from flask import current_app

app = flask.Flask(__name__)
app.config.from_object('amundsen_application.config.LocalConfig')


class ActionLogTest(unittest.TestCase):

    def test_metrics_build(self) -> None:
        # with patch.object(current_app, 'config'):
        with app.test_request_context():
            func_name = 'search'
            metrics = action_log._build_metrics(func_name, 'dummy', 777, foo='bar')

            expected = {'command': 'search',
                        'host_name': socket.gethostname(),
                        'pos_args_json': '["dummy", 777]',
                        'keyword_args_json': '{"foo": "bar"}',
                        'user': getpass.getuser()}

            for k, v in expected.items():
                self.assertEqual(v, metrics.get(k))

            self.assertTrue(metrics.get('start_epoch_ms') <= get_epoch_millisec())  # type: ignore

    def test_fail_function(self) -> None:
        """
        Actual function is failing and fail needs to be propagated.
        :return:
        """
        with app.test_request_context(), self.assertRaises(NotImplementedError):
            fail_func()

    def test_success_function(self) -> None:
        """
        Test success function but with failing callback.
        In this case, failure should not propagate.
        :return:
        """
        with app.test_request_context(), fail_action_logger_callback():
            success_func()


@contextmanager
def fail_action_logger_callback() -> Generator:
    """
    Adding failing callback and revert it back when closed.
    :return:
    """
    tmp = action_log_callback.__pre_exec_callbacks[:]

    def fail_callback(_action_callback: Any) -> None:
        raise NotImplementedError

    action_log_callback.register_pre_exec_callback(fail_callback)
    yield
    action_log_callback.__pre_exec_callbacks = tmp


@action_logging
def fail_func() -> None:
    raise NotImplementedError


@action_logging
def success_func() -> None:
    pass


if __name__ == '__main__':
    unittest.main()



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import os
from amundsen_application import create_app

application = create_app(
    config_module_class=os.getenv('FRONTEND_SVC_CONFIG_MODULE_CLASS') or 'amundsen_application.config.LocalConfig')

if __name__ == '__main__':
    application.run(host='0.0.0.0')



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import unittest

from http import HTTPStatus
from typing import Dict, List

from flask import jsonify, make_response, Response

from amundsen_application import create_app
from amundsen_application.api.exceptions import MailClientNotImplemented
from amundsen_application.api.utils.notification_utils import get_mail_client, \
    get_notification_html, get_notification_subject, send_notification, NotificationType
from amundsen_application.base.base_mail_client import BaseMailClient

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')
local_no_notification_app = create_app('amundsen_application.config.TestNotificationsDisabledConfig', 'tests/templates')


class MockMailClient(BaseMailClient):
    def __init__(self, status_code: int, recipients: List = []) -> None:
        self.status_code = status_code

    def send_email(self,
                   sender: str = None,
                   recipients: List = [],
                   subject: str = None,
                   text: str = None,
                   html: str = None,
                   optional_data: Dict = {}) -> Response:
        return make_response(jsonify({}), self.status_code)


class NotificationUtilsTest(unittest.TestCase):
    def setUp(self) -> None:
        self.mock_table_key = 'db://cluster.schema/table'

    def test_validate_resource_path_none(self) -> None:
        """
        Test Exception is raised if resource_path is None
        :return:
        """
        with local_app.app_context():
            self.assertRaises(Exception,
                              get_notification_html,
                              notification_type=NotificationType.OWNER_REMOVED,
                              options={'resource_name': 'testtable'},
                              sender='test@test.com')

    def test_validate_resource_path_bad_syntax(self) -> None:
        """
        Test Exception is raised if resource_path violates leading '/' syntax
        :return:
        """
        with local_app.app_context():
            self.assertRaises(Exception,
                              get_notification_html,
                              notification_type=NotificationType.OWNER_REMOVED,
                              options={'resource_name': 'testtable', 'resource_path': 'testpath'},
                              sender='test@test.com')

    def test_get_notification_html_bad_base_url(self) -> None:
        """
        Test Exception is raised if configured FRONTEND_BASE violates no trailing '/' syntax
        :return:
        """
        with local_app.app_context():
            temp = local_app.config['FRONTEND_BASE']
            local_app.config['FRONTEND_BASE'] = 'garbagetest_rewrite_file_to_setup_teardown_each_case/'
            self.assertRaises(Exception,
                              get_notification_html,
                              notification_type=NotificationType.OWNER_ADDED,
                              options={'resource_name': 'testtable', 'resource_path': '/testpath'},
                              sender='test@test.com')
            local_app.config['FRONTEND_BASE'] = temp

    def test_get_notification_html_no_resource_name(self) -> None:
        """
        Test Exception is raised if resource_name is not provided
        :return:
        """
        with local_app.app_context():
            self.assertRaises(Exception,
                              get_notification_html,
                              notification_type=NotificationType.OWNER_ADDED,
                              options={'resource_path': '/testpath'},
                              sender='test@test.com')

    def test_get_notification_html_unsupported_type(self) -> None:
        """
        Test Exception is raised if notification_type is not supported
        :return:
        """
        with local_app.app_context():
            self.assertRaises(Exception,
                              get_notification_html,
                              notification_type='invalid_type',
                              options={'resource_name': 'testtable', 'resource_path': '/testpath'},
                              sender='test@test.com')

    def test_get_notification_html_added_success(self) -> None:
        """
        Test successful generation of html for 'added' notification email
        :return:
        """
        test_sender = 'test@test.com'
        test_options = {'resource_name': 'testtable', 'resource_path': '/testpath'}

        with local_app.app_context():
            html = get_notification_html(notification_type=NotificationType.OWNER_ADDED,
                                         options=test_options,
                                         sender=test_sender)
        expectedHTML = ('Hello,<br/><br/>You have been added to the owners list of the '
                        '<a href="http://0.0.0.0:5000/testpath?source=notification">testtable</a>'
                        ' dataset by test@test.com.<br/><br/>What is expected of you?<br/>As an owner, you take an '
                        'important part in making sure that the datasets you own can be used as swiftly as possible '
                        'across the company.<br/>Make sure the metadata is correct and up to date.<br/><br/>If you '
                        'think you are not the best person to own this dataset and know someone who might be, please '
                        'contact this person and ask them if they want to replace you. It is important that we keep '
                        'multiple owners for each dataset to ensure continuity.<br/><br/>Thanks,<br/>Amundsen Team')
        self.assertEqual(html, expectedHTML)

    def test_get_notification_html_removed_success(self) -> None:
        """
        Test successful generation of html for 'removed' notification email
        :return:
        """
        test_sender = 'test@test.com'
        test_options = {'resource_name': 'testtable', 'resource_path': '/testpath'}

        with local_app.app_context():
            html = get_notification_html(notification_type=NotificationType.OWNER_REMOVED,
                                         options=test_options,
                                         sender=test_sender)
        expectedHTML = ('Hello,<br/><br/>You have been removed from the owners list of the '
                        '<a href="http://0.0.0.0:5000/testpath?source=notification">testtable</a> dataset by'
                        ' test@test.com.<br/><br/>If you think you have been incorrectly removed as an owner,'
                        ' add yourself back to the owners list.<br/><br/>Thanks,<br/>Amundsen Team')
        self.assertEqual(html, expectedHTML)

    def test_get_notification_html_requested_success_all_fields(self) -> None:
        """
        Test successful generation of html for 'requested' notification email using
        all required and optional fields
        :return:
        """
        test_sender = 'test@test.com'
        test_options = {
            'resource_name': 'testtable',
            'resource_path': '/testpath',
            'description_requested': True,
            'fields_requested': True,
            'comment': 'Test Comment'
        }
        with local_app.app_context():
            html = get_notification_html(notification_type=NotificationType.METADATA_REQUESTED,
                                         options=test_options,
                                         sender=test_sender)
        expectedHTML = ('Hello,<br/><br/>test@test.com is trying to use '
                        '<a href="http://0.0.0.0:5000/testpath?source=notification">testtable</a>, '
                        'and requests improved table and column descriptions.<br/><br/>test@test.com has included the '
                        'following information with their request:<br/>Test Comment<br/><br/>Please visit the provided '
                        'link and improve descriptions on that resource.<br/><br/>Thanks,<br/>Amundsen Team')
        self.assertEqual(html, expectedHTML)

    def test_get_notification_html_requested_success_table_only(self) -> None:
        """
        Test successful generation of html for 'requested' notification email using
        all required fields and 'description_requested' optional field
        :return:
        """
        test_sender = 'test@test.com'
        test_options = {
            'resource_name': 'testtable',
            'resource_path': '/testpath',
            'description_requested': True,
        }

        with local_app.app_context():
            html = get_notification_html(notification_type=NotificationType.METADATA_REQUESTED,
                                         options=test_options,
                                         sender=test_sender)
        expectedHTML = ('Hello,<br/><br/>test@test.com is trying to use '
                        '<a href="http://0.0.0.0:5000/testpath?source=notification">testtable</a>, and requests '
                        'an improved table description.<br/><br/>Please visit the provided link and improve '
                        'descriptions on that resource.<br/><br/>Thanks,<br/>Amundsen Team')
        self.assertEqual(html, expectedHTML)

    def test_get_notification_html_requested_success_columns_only(self) -> None:
        """
        Test successful generation of html for 'requested' notification email using
        all required fields and 'fields_requested' optional field
        :return:
        """
        test_sender = 'test@test.com'
        test_options = {
            'resource_name': 'testtable',
            'resource_path': '/testpath',
            'fields_requested': True,
        }

        with local_app.app_context():
            html = get_notification_html(notification_type=NotificationType.METADATA_REQUESTED,
                                         options=test_options,
                                         sender=test_sender)
        expectedHTML = ('Hello,<br/><br/>test@test.com is trying to use '
                        '<a href="http://0.0.0.0:5000/testpath?source=notification">testtable</a>, and requests '
                        'improved column descriptions.<br/><br/>Please visit the provided link and improve '
                        'descriptions on that resource.<br/><br/>Thanks,<br/>Amundsen Team')
        self.assertEqual(html, expectedHTML)

    def test_get_notification_html_requested_success_no_optional_options(self) -> None:
        """
        Test successful generation of html for 'requested' notification email using
        all required fields and no optional fields
        :return:
        """
        test_sender = 'test@test.com'
        test_options = {
            'resource_name': 'testtable',
            'resource_path': '/testpath',
        }

        with local_app.app_context():
            html = get_notification_html(notification_type=NotificationType.METADATA_REQUESTED,
                                         options=test_options,
                                         sender=test_sender)
        expectedHTML = ('Hello,<br/><br/>test@test.com is trying to use '
                        '<a href="http://0.0.0.0:5000/testpath?source=notification">testtable</a>, and requests '
                        'more information about that resource.<br/><br/>Please visit the provided link and improve '
                        'descriptions on that resource.<br/><br/>Thanks,<br/>Amundsen Team')
        self.assertEqual(html, expectedHTML)

    def test_get_notification_subject_added(self) -> None:
        """
        Test successful executed of get_notification_subject for the OWNER_ADDED notification type
        :return:
        """
        result = get_notification_subject(notification_type=NotificationType.OWNER_ADDED,
                                          options={'resource_name': 'testtable'})
        self.assertEqual(result, 'You are now an owner of testtable')

    def test_get_notification_subject_removed(self) -> None:
        """
        Test successful executed of get_notification_subject for the OWNER_REMOVED notification type
        :return:
        """
        result = get_notification_subject(notification_type=NotificationType.OWNER_REMOVED,
                                          options={'resource_name': 'testtable'})
        self.assertEqual(result, 'You have been removed as an owner of testtable')

    def test_get_notification_subject_edited(self) -> None:
        """
        Test successful executed of get_notification_subject for the METADATA_EDITED notification type
        :return:
        """
        result = get_notification_subject(notification_type=NotificationType.METADATA_EDITED,
                                          options={'resource_name': 'testtable'})
        self.assertEqual(result, 'Your dataset testtable\'s metadata has been edited')

    def test_get_notification_subject_requested(self) -> None:
        """
        Test successful executed of get_notification_subject for the METADATA_REQUESTED notification type
        :return:
        """
        result = get_notification_subject(notification_type=NotificationType.METADATA_REQUESTED,
                                          options={'resource_name': 'testtable'})
        self.assertEqual(result, 'Request for metadata on testtable')

    def test_get_notification_subject_unsupported_type(self) -> None:
        """
        Test Exception is raised if notification_type is not supported
        :return:
        """
        with local_app.app_context():
            self.assertRaises(Exception,
                              get_notification_subject,
                              notification_type='invalid_type',
                              options={'resource_name': 'testtable'})

    def test_get_mail_client_success(self) -> None:
        """
        Test get_mail_client returns the configured mail client if one is configured
        :return:
        """
        with local_app.app_context():
            local_app.config['MAIL_CLIENT'] = unittest.mock.Mock()
            self.assertEqual(get_mail_client(), local_app.config['MAIL_CLIENT'])

    def test_get_mail_client_error(self) -> None:
        """
        Test get_mail_client raised MailClientNotImplemented if no mail client is configured
        :return:
        """
        with local_app.app_context():
            self.assertRaises(MailClientNotImplemented, get_mail_client)

    def test_send_notification_disabled(self) -> None:
        """
        Test send_notification fails gracefully if notifications are not enabled
        :return:
        """
        with local_no_notification_app.app_context():
            response = send_notification(
                notification_type=NotificationType.OWNER_ADDED,
                options={},
                recipients=['test@test.com'],
                sender='test2@test.com'
            )
            self.assertEqual(response.status_code, HTTPStatus.ACCEPTED)

    @unittest.mock.patch('amundsen_application.api.utils.notification_utils.get_notification_html')
    @unittest.mock.patch('amundsen_application.api.utils.notification_utils.get_notification_subject')
    @unittest.mock.patch('amundsen_application.api.utils.notification_utils.get_mail_client')
    def test_send_notification_success(self, get_mail_client, get_notif_subject, get_notif_html) -> None:
        """
        Test successful execution of send_notification
        :return:
        """
        status_codes = [HTTPStatus.OK, HTTPStatus.ACCEPTED]

        for status_code in status_codes:
            with self.subTest():
                with local_app.app_context():
                    get_mail_client.return_value = MockMailClient(status_code=status_code)
                    get_notif_subject.return_value = 'Test Subject'
                    get_notif_html.return_value = '<div>test html</div>'

                    test_recipients = ['test@test.com']
                    test_sender = 'test2@test.com'
                    test_notification_type = NotificationType.OWNER_ADDED
                    test_options = {}

                    response = send_notification(
                        notification_type=test_notification_type,
                        options=test_options,
                        recipients=test_recipients,
                        sender=test_sender
                    )

                    get_mail_client.assert_called
                    get_notif_subject.assert_called_with(notification_type=test_notification_type,
                                                         options=test_options)
                    get_notif_html.assert_called_with(notification_type=test_notification_type,
                                                      options=test_options,
                                                      sender=test_sender)
                    self.assertTrue(200 <= response.status_code <= 300)

    @unittest.mock.patch('amundsen_application.api.utils.notification_utils.get_notification_html')
    @unittest.mock.patch('amundsen_application.api.utils.notification_utils.get_notification_subject')
    @unittest.mock.patch('amundsen_application.api.utils.notification_utils.get_mail_client')
    def test_remove_sender_from_notification(self, get_mail_client, get_notif_subject, get_notif_html) -> None:
        """
        Test sender is removed if they exist in recipients
        :return:
        """
        with local_app.app_context():
            mock_client = MockMailClient(status_code=HTTPStatus.OK)
            mock_client.send_email = unittest.mock.Mock()
            get_mail_client.return_value = mock_client

            mock_subject = 'Test Subject'
            get_notif_subject.return_value = mock_subject

            mock_html = '<div>test html</div>'
            get_notif_html.return_value = mock_html

            test_sender = 'test@test.com'
            test_recipients = [test_sender, 'test2@test.com']
            test_notification_type = NotificationType.OWNER_ADDED
            test_options = {}
            expected_recipients = ['test2@test.com']

            send_notification(
                notification_type=test_notification_type,
                options=test_options,
                recipients=test_recipients,
                sender=test_sender
            )
            mock_client.send_email.assert_called_with(
                html=mock_html,
                subject=mock_subject,
                optional_data={'email_type': test_notification_type},
                recipients=expected_recipients,
                sender=test_sender
            )

    def test_no_recipients_for_notification(self) -> None:
        """
        Test 200 response with appropriate message if no recipients exist
        :return:
        """
        with local_app.app_context():
            response = send_notification(
                notification_type=NotificationType.OWNER_ADDED,
                options={},
                recipients=[],
                sender='test@test.com'
            )
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertEqual(data.get('msg'), 'No valid recipients exist for notification, notification was not sent.')
